package cn.com.ty.lift.business.annualinspection.service;

import cn.com.ty.lift.business.annualinspection.dto.InspectionRequest;
import cn.com.ty.lift.business.annualinspection.dto.InspectionResponse;
import cn.com.ty.lift.business.annualinspection.entity.AnnualInspection;
import cn.com.ty.lift.business.annualinspection.mapper.AnnualInspectionMapper;
import cn.com.ty.lift.business.common.Values;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.LiftHistory;
import cn.com.ty.lift.business.library.dao.entity.PlatformCompanyLiftRelevance;
import cn.com.ty.lift.business.library.service.LiftHistoryService;
import cn.com.ty.lift.business.library.service.LiftService;
import cn.com.ty.lift.business.library.service.PlatformCompanyLiftRelevanceService;
import cn.com.ty.lift.business.maintenance.service.MaintenancePlanService;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.time.LocalDate;
import java.time.LocalDateTime;

/**
 * <p>
 * 年检记录 服务实现类
 * </p>
 *
 * @author wcz
 * @since 2019-12-10
 */
@Service
@AllArgsConstructor
public class AnnualInspectionService extends ServiceImpl<AnnualInspectionMapper, AnnualInspection> {

    private MaintenancePlanService maintenancePlanService;
    private LiftService liftService;
    private LiftHistoryService liftHistoryService;
    private PlatformCompanyLiftRelevanceService platformCompanyLiftRelevanceService;

    /**
     * 年检列表中根据条件分页查询记录
     * @param request
     * @return
     */
    public IPage<InspectionResponse> pageByCondition(InspectionRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<InspectionResponse> page = new Page<>(pageNum,pageSize);
        return baseMapper.pageByCondition(page,request);
    }

    /**
     * 年检列表中根据条件计算记录总条数
     * @param request
     * @return
     */
    public long countByCondition(InspectionRequest request) {
        return baseMapper.countByCondition(request);
    }

    /**
     * 根据id查询单个年检记录的详情，带全部关联信息
     * @param request
     * @return
     */
    public InspectionResponse selectByIdWithInfo(InspectionRequest request) {
        return baseMapper.selectByIdWithInfo(request);
    }

    /**
     * 根据用户id查询关联的年检
     * @param request
     * @return
     */
    public IPage<InspectionResponse> pageByUser(InspectionRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<InspectionResponse> page = new Page<>(pageNum,pageSize);
        return baseMapper.pageByUser(page,request);
    }

    /**
     * 阶段2 修改电梯的业务状态为年检中
     * 阶段3 如果合格，修改电梯的业务状态为正常，
     * 完成年检，如果是不合格，清空维保计划，如果是合格，直接更新
     * @param entity
     * @return
     */
    @Transactional
    public boolean finish(AnnualInspection entity){
        boolean up = saveOrUpdate(entity);
        if (!up){
            return false;
        }
        Integer stepStatus = entity.getStepStatus();
        Integer stepName = entity.getStepName();
        //阶段2 修改电梯的业务状态为年检中
        if(Values.Ai.stepTwo(stepName)){
            PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(entity.getMtCompanyId(),entity.getLiftId());
            if(null != relevance){
                relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.ANNUAL_INSPECTION.getCode());
                boolean re = platformCompanyLiftRelevanceService.updateById(relevance);
                if(re){
                    return true;
                }else{
                    //强制手动事务回滚
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return false;
                }
            }
        }

        //阶段3 如果合格，修改电梯的业务状态为正常，
        if(Values.Ai.stepThree(stepName)){
            PlatformCompanyLiftRelevance relevance = platformCompanyLiftRelevanceService.findByMtCompanyAndLift(entity.getMtCompanyId(),entity.getLiftId());
            if(null != relevance){
                relevance.setLiftCompanyStatus(CommonEnum.LiftStatus.NORMAL.getCode());
                boolean re = platformCompanyLiftRelevanceService.updateById(relevance);
                if(re){
                    return true;
                }else{
                    //强制手动事务回滚
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return false;
                }
            }
        }
        //todo 不合格：流程结束，电梯扭转到不合格状态列表，电梯停保，清理维保计划记录
        if(Values.Ai.statusCertificateReportUnqualified(stepStatus)) {
            boolean mp = maintenancePlanService.cleanPlan(entity.getMtCompanyId(), entity.getLiftId());
            if (mp) {
                return true;
            } else {
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }
        //todo:合格：修改电梯的年检时间lift-annual_inspection_date，增加lift_history记录
        if(Values.Ai.statusCertificateReportQualified(stepStatus)){
            Lift lift = liftService.getById(entity.getLiftId());
            if(null == lift){
                return false;
            }
            LocalDate oldInspectionDate = lift.getAnnualInspectionDate();
            lift.setAnnualInspectionDate(entity.getNextInspectionTime());
            boolean li = liftService.updateById(lift);
            boolean lh = false;
            if(li){
                LiftHistory liftHistory = new LiftHistory();
                liftHistory.setId(IdWorker.getId());
                liftHistory.setMtCompanyId(entity.getMtCompanyId());
                liftHistory.setLiftId(entity.getLiftId());
                liftHistory.setCode(lift.getLiftCode());
                liftHistory.setDescription("修改电梯年检日期");
                liftHistory.setContent(StrUtil.format("电梯年检日期由:{}修改成{}",oldInspectionDate,entity.getNextInspectionTime()));
                //fixme:获取当前登录用户
                liftHistory.setOperatorId(1l);
                liftHistory.setOperateDate(LocalDateTime.now());
                lh = liftHistoryService.save(liftHistory);
            }
            if(li && lh){
                return true;
            }else{
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return false;
            }
        }
        return true;
    }
}
