package cn.com.ty.lift.business.common;

import net.sourceforge.pinyin4j.PinyinHelper;

/**
 * 拼音工具类
 *
 * @author wcz
 * @date 2018年5月9日
 */
public class PinYinUtils {
    /**
     * 获取汉字首字母的方法。如： 张三 --> zs
     * 说明：暂时解决不了多音字的问题，只能使用取多音字的第一个音的方案
     *
     * @param chinese 汉子字符串
     * @return 大写汉子首字母; 如果都转换失败,那么返回null
     */
    public static String toFirstChar(String chinese) {
        return toFirstChar(chinese,false);
    }

    /**
     * 获取汉字首字母的方法。如： 张三 --> zs/ZS
     * 说明：暂时解决不了多音字的问题，只能使用取多音字的第一个音的方案
     *
     * @param chinese 汉子字符串
     * @param toUpperCase true -> 转换成大写; false-> 默认小写
     * @return 大写汉子首字母; 如果都转换失败,那么返回null
     */
    public static String toFirstChar(String chinese, boolean toUpperCase) {
        String result = null;
        if(null == chinese || chinese.equals("")){
            return result;
        }
        char[] charArray = chinese.toCharArray();
        StringBuffer sb = new StringBuffer();
        for (char ch : charArray) {
            // 逐个汉字进行转换， 每个汉字返回值为一个String数组（因为有多音字）
            String[] stringArray = PinyinHelper.toHanyuPinyinStringArray(ch);
            if (null != stringArray) {
                sb.append(stringArray[0].charAt(0));
            }
        }
        if (sb.length() > 0) {
            result = sb.toString();
        }
        if(toUpperCase){
            result.toUpperCase();
        }
        return result;
    }

    /**
     * 获取汉字拼音的方法。如： 张三 --> zhangsan
     * 说明：暂时解决不了多音字的问题，只能使用取多音字的第一个音的方案
     *
     * @param chinese 汉子字符串
     * @return 汉字拼音; 如果都转换失败,那么返回null
     */
    public static String toPinYin(String chinese) {
        return toPinYin(chinese,false);
    }

    /**
     * 获取汉字拼音的方法。如： 张三 --> zhangsan/ZHANGSAN
     * 说明：暂时解决不了多音字的问题，只能使用取多音字的第一个音的方案
     *
     * @param chinese 汉子字符串
     * @param toUpperCase true -> 转换成大写; false-> 默认小写
     * @return 汉字拼音; 如果都转换失败,那么返回null
     */
    public static String toPinYin(String chinese,boolean toUpperCase) {
        String result = null;
        if(null == chinese || chinese.equals("")){
            return result;
        }
        char[] charArray = chinese.toCharArray();
        StringBuffer sb = new StringBuffer();
        for (char ch : charArray) {
            // 逐个汉字进行转换， 每个汉字返回值为一个String数组（因为有多音字）
            String[] stringArray = PinyinHelper.toHanyuPinyinStringArray(ch);
            if (null != stringArray) {
                // 把第几声这个数字给去掉
                sb.append(stringArray[0].replaceAll("\\d", ""));
            }
        }
        if (sb.length() > 0) {
            result = sb.toString();
        }
        if(toUpperCase){
            result.toUpperCase();
        }
        return result;
    }

    public static void main(String[] args) {
        System.out.println(PinYinUtils.toFirstChar("张三"));
        System.out.println(PinYinUtils.toPinYin("张三"));
    }
}