package cn.com.ty.lift.business.common;

/**
 * @author wcz
 * @Description: 定义系统的一些常量
 * @date 2019/12/31 9:18
 */
public interface Values {

    //======================年检相关状态值和判断方法======================================

    interface Ai{
        /**
         * 年检未开始0
         */
        int STEP_NOT_START = 0;

        /**
         * 判断年检是否开始
         * @param step
         * @return
         */
        static boolean notStart(int step){
            return STEP_NOT_START == step;
        }
        /**
         * 年检 第一阶段
         */
        int STEP_ONE = 1;

        /**
         * 判断是否为年检阶段一
         * @param step
         * @return
         */
        static boolean stepOne(int step){
            return STEP_ONE == step;
        }
        /**
         * 年检 第二阶段
         */
        int STEP_TWO = 2;

        /**
         * 判断是否为年检阶段二
         * @param step
         * @return
         */
        static boolean stepTwo(int step){
            return STEP_TWO == step;
        }
        /**
         * 年检 第三阶段
         */
        int STEP_THREE = 3;

        /**
         * 判断是否为年检阶段三
         * @param step
         * @return
         */
        static boolean stepThree(int step){
            return STEP_THREE == step;
        }
        /**
         * 年检 第四阶段
         */
        int STEP_FOUR = 4;

        /**
         * 判断是否为年检阶段四
         * @param step
         * @return
         */
        static boolean stepFour(int step){
            return STEP_FOUR == step;
        }

        /**
         * 0 年检计划初始状态
         */
        int STATUS_NOT_START = 0;

        static boolean statusNotStart(int status){
            return STEP_NOT_START == status;
        }
        /**
         * 1  维保工确认年检计划
         */
        int STATUS_CONFIRM = 1;
        String confirm = "请确认年检计划";

        static boolean statusConfirm(int status){
            return STATUS_CONFIRM == status;
        }
        /**
         * 2  维保工上传自检报告,选择自检时间 或者 文员设置 是否检查限速器和荷载年检设置，
         *             文员也可以上传自检报告或者确认自检报告和自检时间，自检时间以最后设置的为准
         */
        int STATUS_SELFCHECK = 2;

        String selfcheckUpload = "请完善自检报告信息";
        String selfcheckDate = "请设置自检时间";
        String selfcheckReportImg = "请设置自检报告";

        static boolean statusSelfcheck(int status){
            return STATUS_SELFCHECK == status;
        }
        /**
         * 3  企业文员确认现场检验时间
         *             录入人员信息，政府质检，企业质检，联系电话
         */
        int STATUS_CHECK_CONFIRM = 3;
        String checkConfirm = "请确认检验结果";
        String checkDate = "请设置现场检验时间";
        String checkResultImg = "请设置现场检验结果";
        String inspector = "请设置政府质检人员";
        String operator = "请设置公司质检人员";
        String telephone = "请设置人员联系电话";

        static boolean statusCheckConfirm(int status){
            return STATUS_CHECK_CONFIRM == status;
        }
        /**
         * 4	维保工上传检验结果
         */
        int STATUS_CHECK_UPLOAD = 4;

        static boolean statusCheckUpload(int status){
            return STATUS_CHECK_UPLOAD == status;
        }
        /**
         * 5	企业文员确认检验结果-合格 --> 不能算年检超期
         */
        int STATUS_CHECK_QUALIFIED = 5;

        static boolean statusCheckQualified(int status){
            return STATUS_CHECK_QUALIFIED == status;
        }
        /**
         * 6	企业文员确认检验结果-整改 --> 回到第二阶段
         */
        int STATUS_CHECK_RECTIFICATION = 6;

        static boolean statusCheckRectification(int status){
            return STATUS_CHECK_RECTIFICATION == status;
        }
        /**
         * 7	企业文员确认检验结果-不合格 --> 可以算年检超期
         */
        int STATUS_CHECK_UNQUALIFIED = 7;

        static boolean statusCheckUnqualified(int status){
            return STATUS_CHECK_UNQUALIFIED == status;
        }
        /**
         * 8	维保工上传年检报告和合格证
         */
        int STATUS_CERTIFICATE_REPORT_UPLOAD = 8;
        String certificateReportUpload = "请上传年检报告和合格证";
        String certificateUpload = "请设置年检报告";
        String reportUpload = "请设置合格证";

        static boolean statusCertificateReportUpload(int status){
            return STATUS_CERTIFICATE_REPORT_UPLOAD == status;
        }
        /**
         * 9	企业文员确认年检报告和合格证-合格
         */
        int STATUS_CERTIFICATE_REPORT_QUALIFIED = 9;
        String nextInspectionTime = "请设置下次年检时间";

        static boolean statusCertificateReportQualified(int status){
            return STATUS_CERTIFICATE_REPORT_QUALIFIED == status;
        }
        /**
         * 10	企业文员确认年检报告和合格证-不合格 -->停用电梯
         */
        int STATUS_CERTIFICATE_REPORT_UNQUALIFIED = 10;

        static boolean statusCertificateReportUnqualified(int status){
            return STATUS_CERTIFICATE_REPORT_UNQUALIFIED == status;
        }
        String statusError = "年检状态有误";

        /**
         * 根据年检的状态值返回成年检的状态阶段
         * @param stepStatus
         * @return
         */
        static int status2StepName(Integer stepStatus){
            int stepName;
            switch (stepStatus){
                case STATUS_NOT_START:
                    stepName = 0;
                    break;
                case STATUS_CONFIRM:
                case STATUS_SELFCHECK:
                    stepName = 1;
                    break;
                case STATUS_CHECK_CONFIRM:
                    stepName = 2;
                    break;
                case STATUS_CHECK_UPLOAD:
                case STATUS_CHECK_QUALIFIED:
                case STATUS_CHECK_RECTIFICATION:
                case STATUS_CHECK_UNQUALIFIED:
                    stepName = 3;
                    break;
                case STATUS_CERTIFICATE_REPORT_UPLOAD:
                case STATUS_CERTIFICATE_REPORT_QUALIFIED:
                case STATUS_CERTIFICATE_REPORT_UNQUALIFIED:
                    stepName = 4;
                    break;
                default:
                    stepName = -1;
                    break;
            }
            return stepName;
        }


        /**
         * 判断年检状态是否处于其他阶段之间 min < middle < max
         * @param middle
         * @param min
         * @param max
         * @return
         */
        static boolean inStatus(int min, int middle,int max){
            return min <= middle && middle <= max;
        }

        /**
         * 判断两个年检状态是否一样 one == two
         * @param one
         * @param two
         * @return
         */
        static boolean sameStatus(int one,int two){
            return one == two;
        }

        /**
         * 判断年检中一个状态是否为另一个状态的下一个状态
         * @param front
         * @param behind
         * @return
         */
        static boolean nextStatus(int front,int behind){
            return front + 1 == behind;
        }

        /**
         * 判断年检中两个状态相同或者为下一个状态
         * @param front
         * @param behind
         * @return
         */
        static boolean sameNext(int front,int behind){
            return front == behind || front + 1 == behind;
        }

        //==============================================================================
    }

    interface Er{
        //===================急修相关状态值和判断方法======================================
        /**
         * 急修状态（-1 暂停中）
         */
        int STATUS_STOPING = -1;

        static boolean statusStoping(int status){
            return STATUS_STOPING == status;
        }
        /**
         * 急修状态（0 待修理）
         */
        int STATUS_TO_REPAIRED = 0;

        static boolean statusToRepaired(int status){
            return STATUS_TO_REPAIRED == status;
        }
        /**
         * 急修状态（1 修理中）
         */
        int STATUS_REPAIRING = 1;

        static boolean statusRepairing(int status){
            return STATUS_REPAIRING == status;
        }
        /**
         * 急修状态（2 完成）
         */
        int STATUS_FINISH = 2;

        static boolean statusFinish(int status){
            return STATUS_FINISH == status;
        }
        /**
         * 急修状态（3 关闭）
         */
        int STATUS_CLOSE = 3;
        static boolean statusClose(int status){
            return STATUS_CLOSE == status;
        }
        String haveUnfinisedRepair = "该电梯存在未完成的急修，无法创建";
        String repairHadTaked = "急修已经接单，不能转派";
        String missingReason = "缺少原因描述";
        String missingHandleDate = "缺少处理时间";
        String missionPosition = "缺少地理位置信息";
        String mustToRepair = "待急修才能操作";
        String mustInRepairing = "急修中才能操作";
        String safetyConfirm = "请完成安全确认";
        String missingCost = "缺少收费项";
        String missingFault = "缺少故障信息";
        String missingSign = "请签名";
        String mustFinish = "急修完成后才能操作";
        String hadEvaluate = "急修已经评价了";
        String missingImage = "缺少急修图片";
        String missingScore = "请评价打分";
        String incorrectAmount = "金额有误";

        //==============================================================================
    }

    interface Upload{
        //===================上传相关状态值和判断方法======================================
        /**
         * 图片上传最大大小10M * 1024 * 1024
         */
        long maxSizePic = 10 << 20;
        /**
         * 文件上传最大大小50M * 1024 * 1024
         */
        long maxSizeFile = 50 << 20;
        /**
         * 视频上传最大大小100M * 1024 * 1024
         */
        long maxSizeVideo = 100 << 20;

        /**
         * 上传图片文件类型
         */
        String[] pics = {".jpg",".jpeg", ".png", ".bmp", ".gif"};
        /**
         * 上传其他文件类型
         */
        String[] files = {".pdf",".txt",".rar",".zip",".7z", ".xls", ".xlsx", ".doc",".docx",".ppt",".pptx"};
        /**
         * 上传视频文件类型
         */
        String[] videos = {".mov",".mp4", ".avi",".mpg",".mpeg",".rm",".rmvb",".wmv"};

        String missingFileData = "没有文件数据";

        String fileFail = "上传文件失败";
        //==============================================================================
    }

}
