package cn.com.ty.lift.business.contract.controller;

import cn.com.ty.lift.business.contract.dao.entity.Contracts;
import cn.com.ty.lift.business.contract.dao.entity.ContractsExtend;
import cn.com.ty.lift.business.contract.dao.entity.ContractsHistory;
import cn.com.ty.lift.business.contract.dao.entity.model.ContractDetailRequest;
import cn.com.ty.lift.business.contract.dao.entity.model.ContractRequest;
import cn.com.ty.lift.business.contract.dao.entity.model.ContractsHistoryRequest;
import cn.com.ty.lift.business.contract.dao.entity.model.PaymentRequest;
import cn.com.ty.lift.business.contract.service.ContractService;
import cn.com.ty.lift.business.contract.service.PaymentService;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * @author bieao
 * @date 2019/12/07
 * @description 合同管理控制层
 */
@RestController
@RequestMapping("contract")
public class ContractController {

    @Resource
    private ContractService contractService;

    @Resource
    private PaymentService paymentService;


    /**
     * @param request 合同列表查询条件
     * @return RestResponse 合同分页列表结果
     * @description 查询合同列表
     * @date 2019/12/7 11:53 AM
     */
    @PostMapping("list")
    public RestResponse list(@Valid @RequestBody ContractRequest request) {
        IPage<Contracts> page = contractService.list(request);
        if (CollUtil.isEmpty(page.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(page, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 合同操作记录列表查询条件
     * @return RestResponse 合同操作记录分页列表结果
     * @description 查询合同操作记录列表
     * @date 2019/12/7 11:53 AM
     */
    @PostMapping("history/list")
    public RestResponse list(@RequestBody ContractsHistoryRequest request) {
        IPage<ContractsHistory> page = contractService.list(request);
        if (CollUtil.isEmpty(page.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(page, MessageUtils.get("msg.query.success"));
    }


    /**
     * @param contracts 新增/续签合同数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 新增/续签合同
     * @date 2019/12/7 3:31 PM
     */
    @PostMapping("add")
    public RestResponse add(@Valid @RequestBody ContractsExtend contracts) {
        return contractService.add(contracts);
    }


    /**
     * @param contracts 修改合同数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 合同变更
     * @date 2019/12/7 3:31 PM
     */
    @PostMapping("modify")
    public RestResponse modify(@RequestBody ContractsExtend contracts) {
       return contractService.modify(contracts);
    }

    /**
     * @param request 合同id
     * @return RestResponse 合同详情
     * @description 查询合同详情
     * @date 2019/12/6 4:46 PM
     */
    @PostMapping("detail")
    public RestResponse detail(@Valid @RequestBody ContractDetailRequest request) {
        Contracts contracts = contractService.detail(request.getId());
        if (ObjectUtil.isEmpty(contracts)) {
            return RestResponse.success();
        }
        return RestResponse.success(contracts, MessageUtils.get("msg.query.detail.success"));
    }

    /**
     * @param request paymentList 收款信息列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 批量修改收款信息
     * @date 2019/12/11 3:50 PM
     */
    @PostMapping("payment/batch/update")
    public RestResponse modifyBatchById(@RequestBody PaymentRequest request) {
        boolean result = paymentService.modifyBatchById(request.getPaymentList());
        if (result) {
            return RestResponse.success(null, MessageUtils.get("msg.modify.batch.success"));
        }
        return RestResponse.fail(MessageUtils.get("msg.modify.batch.fail"));
    }

}
