package cn.com.ty.lift.business.library.controller;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.model.request.*;
import cn.com.ty.lift.business.library.dao.entity.model.response.LiftExtendResponse;
import cn.com.ty.lift.business.library.dao.entity.model.response.LiftResponse;
import cn.com.ty.lift.business.library.service.LiftService;
import cn.com.ty.lift.business.library.service.PlatformCompanyLiftRelevanceService;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.project.service.ProjectLiftRelevanceService;
import cn.com.ty.lift.common.base.ExportRequest;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * @author bieao
 * @date 2019/11/27
 * @description 电梯控制层
 */
@RestController
@RequestMapping("lift")
public class LiftController {

    @Resource
    private LiftService liftService;

    @Resource
    private PlatformCompanyLiftRelevanceService platformService;

    @Resource
    private ProjectLiftRelevanceService projectLiftRelevanceService;

    /**
     * @param request 电梯列表查询条件
     * @return RestResponse 电梯分页列表结果
     * @description 查询电梯列表
     * @date 2019/11/27 10:03 AM
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody LiftRequest request) {
        IPage<LiftResponse> page = liftService.list(request);
        if (CollUtil.isEmpty(page.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(page, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 电梯列表查询条件
     * @return RestResponse 项目下电梯列表
     * @description 查询项目下电梯列表
     * @date 2019/12/17 11:04 AM
     */
    @PostMapping("project/list")
    public RestResponse liftProjectList(@Valid @RequestBody ProjectLiftRequest request) {
        IPage<LiftResponse> page = liftService.liftProjectList(request);
        if (CollUtil.isEmpty(page.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(page, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 选择电梯列表查询条件
     * @return IPage<LiftResponse> 选择电梯列表
     * @description 选择电梯
     * @date 2019/12/30 10:42 AM
     */
    @PostMapping("choose")
    public RestResponse choose(@Valid @RequestBody LiftChooseRequest request) {
        IPage<LiftResponse> page = liftService.choose(request);
        if (CollUtil.isEmpty(page.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(page, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 项目id，项目电梯关联id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 批量移动电梯
     * @date 2020/1/7 7:57 下午
     */
    @PostMapping("batch/move")
    public RestResponse move(@Valid @RequestBody LiftBatchMoveRequest request){
        boolean result = projectLiftRelevanceService.move(request);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.modify.batch.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.modify.batch.success"));
    }

    /**
     * @param request 项目电梯关联列表请求
     * @return 1.成功, 0.失败, 消息描述
     * @description 批量插入项目电梯关联数据
     * @date 2020/1/2 2:59 下午
     */
    @PostMapping("batch/insert")
    public RestResponse insertBatch(@Valid @RequestBody ProjectLiftBatchRequest request) {
        boolean result = projectLiftRelevanceService.insertBatch(request.getList());
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.add.batch.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.add.batch.success"));
    }

    /**
     * @param request 电梯id
     * @return RestResponse 电梯详情
     * @description 查询电梯详情
     * @date 2019/12/6 10:51 AM
     */
    @PostMapping("detail")
    public RestResponse detail(@Valid @RequestBody LiftDetailRequest request) {
        LiftExtendResponse response = liftService.detail(request.getId());
        if (ObjectUtil.isEmpty(response)) {
            return RestResponse.success();
        }
        return RestResponse.success(response, MessageUtils.get("msg.query.detail.success"));
    }

    /**
     * @param request 公司id，电梯注册代码
     * @return RestResponse 判断结果
     * @description 电梯校验
     * @date 2019/12/20 2:31 PM
     */
    @PostMapping("preJudgment")
    public RestResponse preJudgment(@RequestBody LiftExtensionRequest request) {
        return liftService.preJudgment(request.getMtCompanyId(), request.getCode());
    }

    /**
     * @param request 新增电梯数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 新增电梯
     * @date 2019/11/27 10:03 AM
     */
    @PostMapping("add")
    public RestResponse add(@RequestBody LiftExtensionRequest request) {
        return liftService.add(request);
    }

    /**
     * @param lift 修改电梯数据项
     * @return 1.成功, 0.失败, 消息描述
     * @description 修改电梯
     * @date 2019/11/27 10:03 AM
     */
    @PostMapping("modify")
    public RestResponse modify(@RequestBody Lift lift) {
       return liftService.modify(lift);
    }

    /**
     * @param request updateLiftList 项目id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 电梯批量锁定、解锁
     * @date 2019/11/27 10:03 AM
     */
    @PostMapping("lockOrUnlock")
    public RestResponse lockOrUnlock(@Valid @RequestBody LiftLockRequest request) {
        return projectLiftRelevanceService.liftLockOrUnLock(request);
    }


    /**
     * @param projectLiftRelevance 关联表主键id
     * @return 1.成功, 0.失败, 消息描述
     * @description 逻辑删除电梯
     * @date 2019/11/27 10:03 AM
     */
    @PostMapping("logicDelete")
    public RestResponse logicDelete(@RequestBody ProjectLiftRelevance projectLiftRelevance) {
        return projectLiftRelevanceService.logicDelete(projectLiftRelevance);
    }

    /**
     * @param request idList 电梯id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 更新电梯状态为停保或恢复为正常
     * @date 2019/12/5 5:24 PM
     */
    @PostMapping("stop/recover")
    public RestResponse stopOrRecover(@RequestBody PlatformCompanyRequest request) {
        return platformService.stopOrRecover(request);
    }

    /**
     * @param request idList 电梯id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 更新电梯状态为停保
     * @date 2019/12/5 5:24 PM
     */
    @PostMapping("batch/stop/insurance")
    public RestResponse batchStopInsurance(@Valid @RequestBody PlatformCompanyBatchRequest request) {
        return platformService.batchStopInsurance(request);
    }

    /**
     * @param request 导出电梯数据列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 导出电梯
     * @date 2019/11/28 2:19 PM
     */
    @PostMapping("export")
    public void export(@RequestBody ExportRequest request) {
        liftService.export(request);
    }
}
