package cn.com.ty.lift.business.library.service;

import cn.com.ty.lift.business.common.Judge;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.PlatformCompanyLiftRelevance;
import cn.com.ty.lift.business.library.dao.entity.model.LiftImportModel;
import cn.com.ty.lift.business.library.dao.entity.model.LiftProjectModel;
import cn.com.ty.lift.business.library.dao.entity.model.request.LiftChooseRequest;
import cn.com.ty.lift.business.library.dao.entity.model.request.LiftExtensionRequest;
import cn.com.ty.lift.business.library.dao.entity.model.request.LiftRequest;
import cn.com.ty.lift.business.library.dao.entity.model.request.ProjectLiftRequest;
import cn.com.ty.lift.business.library.dao.entity.model.response.LiftExtendResponse;
import cn.com.ty.lift.business.library.dao.entity.model.response.LiftResponse;
import cn.com.ty.lift.business.library.dao.mapper.LiftMapper;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.project.service.ProjectLiftRelevanceService;
import cn.com.ty.lift.common.base.ExportRequest;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.ty.lift.common.export.ExportUtils;
import cn.com.ty.lift.common.utils.DateUtils;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author bieao
 * @date 2019/11/27
 * @description 电梯业务层
 */
@Slf4j
@Service
public class LiftService extends ServiceImpl<LiftMapper, Lift> {

    @Resource
    private LiftMapper liftMapper;

    @Resource
    private PlatformCompanyLiftRelevanceService platformService;

    @Resource
    private ProjectLiftRelevanceService projectRelevanceService;

    private Map<String, String> paramMap = new HashMap<String, String>() {{
        put("liftCode", "电梯号");
        put("registrationCode", "注册代码");
        put("liftBrand", "电梯品牌");
        put("liftType", "电梯类型");
        put("devicePosition", "设备使用地点");
    }};

    /**
     * @param request 电梯列表查询条件
     * @return IPage<LiftResponse> 电梯列表分页结果
     * @description 条件查询电梯档案列表
     * @date 2019/11/27 2:10 PM
     */
    public IPage<LiftResponse> list(LiftRequest request) {
        IPage<LiftResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return liftMapper.findByCondition(page, request);
    }

    /**
     * @param request 电梯列表查询条件
     * @return IPage<LiftResponse> 项目下电梯列表
     * @description 查询项目下电梯列表
     * @date 2019/12/17 11:04 AM
     */
    public IPage<LiftResponse> liftProjectList(ProjectLiftRequest request) {
        IPage<LiftResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return liftMapper.findLiftListByProjectId(page, request);
    }

    /**
     * @param request 选择电梯列表查询条件
     * @return IPage<LiftResponse> 选择电梯列表
     * @description 选择电梯
     * @date 2019/12/30 10:42 AM
     */
    public IPage<LiftResponse> choose(LiftChooseRequest request) {
        IPage<LiftResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return liftMapper.chooseLiftList(page, request);
    }

    /**
     * @param id 电梯id
     * @return Lift 电梯详情
     * @description 查询电梯详情
     * @date 2019/12/6 10:51 AM
     */
    public LiftExtendResponse detail(Long id) {
        return liftMapper.queryById(id);
    }


    /**
     * @param mtCompanyId      公司id
     * @param registrationCode 注册代码
     * @return RestResponse 判断结果
     * @description 新增电梯前置判断条件
     * @date 2019/12/20 2:31 PM
     */
    public RestResponse preJudgment(Long mtCompanyId, String registrationCode) {
        //查询电梯表是否有该电梯
        Optional<Lift> lift = getOne(registrationCode, null);
        Long liftId = lift.map(Lift::getId).orElse(null);
        if (ObjectUtil.isNotEmpty(liftId)) {
            Map<String, Object> paramMap = new HashMap<>(2);
            paramMap.put("liftId", liftId);
            paramMap.put("mtCompanyId", mtCompanyId);
            //查询该公司下是否有此电梯
            boolean result = platformService.findCompanyLiftExist(paramMap);
            if (result) {
                return RestResponse.fail(MessageUtils.get("msg.lift.exist"));
            } else {
                LiftExtendResponse detail = detail(liftId);
                return RestResponse.success(detail, MessageUtils.get("msg.lift.company.exist"));
            }
        } else {
            return RestResponse.success(null, MessageUtils.get("msg.pre.judge"));
        }
    }

    /**
     * @param registrationCode 电梯注册代码
     * @return 电梯id
     * @description 查询电梯是否存在
     * @date 2019/12/20 11:04 AM
     */
    public Optional<Lift> getOne(String registrationCode, Long liftId) {
        QueryWrapper<Lift> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<Lift> lambdaQueryWrapper = queryWrapper.lambda();
        if (ObjectUtil.isNotEmpty(liftId)) {
            lambdaQueryWrapper.eq(Lift::getId, liftId);
        }
        if (StrUtil.isNotEmpty(registrationCode)) {
            lambdaQueryWrapper.eq(Lift::getRegistrationCode, registrationCode);
        }
        return Optional.ofNullable(getOne(lambdaQueryWrapper));
    }

    /**
     * @param id 电梯id
     * @param mtCompanyId 公司id
     * @return RestResponse 是否成功
     * @description 保存公司电梯关联数据、平台电梯关联数据
     * @date 2020/1/11 4:18 下午
     */
    public RestResponse saveCompanyLiftInfo(Long id, Long mtCompanyId) {
        //新增企业电梯关联表
        PlatformCompanyLiftRelevance ret = platformService.save(id, mtCompanyId);
        if (ObjectUtil.isEmpty(ret)) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        //新增平台电梯关联表
        PlatformCompanyLiftRelevance liftRelevance = platformService.save(id, ApiConstants.PLATFORM_COMPANY_ID);
        if (ObjectUtil.isEmpty(liftRelevance)) {
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.add.success"));
    }

    /**
     * @param request 新增电梯数据项
     * @return RestResponse 状态码和返回消息
     * @description 新增电梯
     * @date 2019/11/27 2:22 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse add(LiftExtensionRequest request) {
        //电梯id
        Long id;
        //获取新增标记 (0:新增,1:查询后带入)
        String mark = request.getMark();
        //公司id
        Long mtCompanyId = request.getMtCompanyId();
        Lift lift = request.getLift();
        /*if (judge(mtCompanyId))
        return RestResponse.fail(ApiConstants.RESULT_ERROR, MessageUtils.get("msg.limit.num"));*/
        RestResponse response = preJudgment(mtCompanyId, lift.getRegistrationCode());
        if (ApiConstants.RESULT_ERROR.equals(response.getStatusCode())) return response;
        //新增标记 (0:新增,1:查询后带入)
        if ("0".equals(mark)) {
            //按当前日期时间戳自动生成电梯编号
            String liftCode = ApiConstants.STRAIGHT_LADDER + DateUtils.generateCode();
            if (CommonEnum.LiftType.STAIRCASE.getCode().equals(Objects.toString(lift.getLiftType()))) {
                lift.setLiftCode(ApiConstants.STAIRCASE + liftCode);
            }
            //新增电梯表
            int liftResult = liftMapper.insert(lift);
            if (liftResult == 0) {
                return RestResponse.fail(MessageUtils.get("msg.add.fail"));
            }
            id = lift.getId();
        } else {
            id = request.getId();
        }
        //新增电梯来源
        String source = request.getSource();
        switch (source) {
            //电梯管理入口
            case "0":
                saveCompanyLiftInfo(id, mtCompanyId);
                break;
            //项目管理入口
            case "1":
                //新增企业电梯关联表
                PlatformCompanyLiftRelevance companyLiftEntry = platformService.save(id, mtCompanyId);
                if (ObjectUtil.isEmpty(companyLiftEntry)) {
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                }
                //新增平台电梯关联表
                PlatformCompanyLiftRelevance relevance = platformService.save(id, ApiConstants.PLATFORM_COMPANY_ID);
                if (ObjectUtil.isEmpty(relevance)) {
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                }
                //新增项目电梯关联表
                boolean result = projectRelevanceService.save(request, companyLiftEntry.getId(), request.getWorkerId());
                if (!result) {
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return RestResponse.fail(MessageUtils.get("msg.add.fail"));
                }
                break;
            default:
                break;
        }
        return RestResponse.success(null, MessageUtils.get("msg.add.success"));
    }

    /**
     * @param lift 修改电梯数据项
     * @return RestResponse 状态码和返回消息
     * @description 修改电梯
     * @date 2019/11/27 2:22 PM
     */
    public RestResponse modify(Lift lift) {
        int result = liftMapper.updateById(lift);
        if (result > 0) {
            return RestResponse.success(null, MessageUtils.get("msg.modify.success"));
        }
        return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
    }

    /**
     * @param request 导出电梯数据列表
     * @return RestResponse 状态码和返回消息
     * @description 导出电梯
     * @date 2019/11/27 2:22 PM
     */
    public void export(ExportRequest request) {
        List<Lift> liftList = liftMapper.selectBatchIds(request.getIds());
        ExportUtils utils = new ExportUtils();
        utils.export(liftList, paramMap, request.getExportPath());
    }

    /**
     * 解析excel导入电梯
     *
     * @param liftImportModels
     * @param mtCompanyId
     * @param projectId
     * @return
     */
    @Transactional
    public RestResponse assemble(List<LiftImportModel> liftImportModels, Long mtCompanyId, Long projectId) {
        Judge.notNull(liftImportModels, "没有数据");
        //存放新建的lift
        List<Lift> lifts = new ArrayList<>();
        //存放新建的project-lift关联信息
        List<ProjectLiftRelevance> projectLiftRelevances = new ArrayList<>();
        //存放新建的platform-company-lift关联信息
        List<PlatformCompanyLiftRelevance> platformCompanyLiftRelevances = new ArrayList<>();

        //根据维保公司和项目id查询
        List<LiftProjectModel> liftProjectModelList = liftMapper.listByCompanyAndProject(mtCompanyId, projectId);
        //根据维保公司和项目查询

        List<ProjectLiftRelevance> projectLiftRelevanceList = projectRelevanceService.findLiftList(mtCompanyId,projectId);

        //根据维保公司
        List<PlatformCompanyLiftRelevance> platformCompanyLiftRelevanceList = platformService.listByCompany(mtCompanyId);

        for (int i = 0; i < liftImportModels.size(); i++) {
            LiftImportModel item = liftImportModels.get(i);
            Lift lift = new Lift();
            //复制解析model中的值到lift中
            BeanUtil.copyProperties(item, lift, true);
            log.info("复制电梯属性后：{}", lift);

            String registrationCode = item.getRegistrationCode();
            LiftProjectModel liftProjectModel = liftProjectModelList.stream().filter(lpm -> (
                    StrUtil.equals(registrationCode, lpm.getRegistrationCode())
            )).distinct().limit(1).findFirst().get();

            if (null != liftProjectModel) {
                //如果已经存在lift，取出id赋值
                lift.setId(liftProjectModel.getLiftId());
            } else {
                //如果不存在,新建id赋值
                lift.setId(IdWorker.getId());
                //加入list中批量插入
                lifts.add(lift);
            }

            PlatformCompanyLiftRelevance platformCompanyLiftRelevance = platformCompanyLiftRelevanceList.stream().filter(pclr -> (
                    pclr.getLiftId() == lift.getId()
            )).distinct().limit(1).findFirst().get();
            if (null == platformCompanyLiftRelevance) {
                //如果不存在就新建
                platformCompanyLiftRelevance = new PlatformCompanyLiftRelevance();
                Long relevanceId = IdWorker.getId();
                //属性赋值
                platformCompanyLiftRelevance.setId(relevanceId);
                platformCompanyLiftRelevance.setMtCompanyId(mtCompanyId);
                platformCompanyLiftRelevance.setLiftId(lift.getId());
                platformCompanyLiftRelevance.setLiftCompanyStatus(CommonEnum.LiftStatus.NORMAL.getCode());
                log.info("新建PlatformCompanyLiftRelevance:{}", platformCompanyLiftRelevance);
                //加入到新建list中批量插入
                platformCompanyLiftRelevances.add(platformCompanyLiftRelevance);
            }

            //fixme:导入电梯时，是否指定项目导入，添加ProjectLiftRelevance关联信息
            ProjectLiftRelevance projectLiftRelevance = projectLiftRelevanceList.stream().filter(plr -> (
                    plr.getLiftId() == lift.getId()
            )).distinct().limit(1).findFirst().get();
            if (null == projectLiftRelevance) {
                projectLiftRelevance = new ProjectLiftRelevance();
                projectLiftRelevance.setMtCompanyId(mtCompanyId);
                projectLiftRelevance.setLiftId(lift.getId());
                projectLiftRelevance.setProjectId(projectId);
                projectLiftRelevance.setRelevanceId(platformCompanyLiftRelevance.getId());
                log.info("新建ProjectLiftRelevance：{}", projectLiftRelevance);
                projectLiftRelevances.add(projectLiftRelevance);
            }
        }

        //批量插入platform-company-lift关联数据
        if (!platformCompanyLiftRelevances.isEmpty()) {
            boolean plat = platformService.saveOrUpdateBatch(platformCompanyLiftRelevances);
            if (!plat) {
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return RestResponse.fail();
            }
        }
        //批量插入project-lift关联数据
        if (!projectLiftRelevances.isEmpty()) {
            boolean proj = projectRelevanceService.saveOrUpdateBatch(projectLiftRelevances);
            if (!proj) {
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return RestResponse.fail();
            }
        }
        //批量插入lift数据
        if (!lifts.isEmpty()) {
            boolean lift = saveOrUpdateBatch(lifts);
            if (!lift) {
                //强制手动事务回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return RestResponse.fail();
            }
        }
        return RestResponse.success("导入成功");
    }

    public List<Lift> groupByRegistrationCode(){
        QueryWrapper<Lift> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<Lift> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.groupBy(Lift::getRegistrationCode);
        lambdaQueryWrapper.having("count(*) > 1");
        lambdaQueryWrapper.select(Lift::getRegistrationCode);
        return list(lambdaQueryWrapper);
    }

    public List<Lift> listByRegistrationCode(List<String> codes){
        QueryWrapper<Lift> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<Lift> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.in(Lift::getRegistrationCode,codes);
        lambdaQueryWrapper.select(Lift::getId,Lift::getRegistrationCode);
        return list(lambdaQueryWrapper);
    }


    /**
     * 去掉lift中registration_code对应的重复数据，同步其他表中lift_id
     * @return
     */
    public boolean clearData() {
        //1 获取同一registration_code对应多个电梯的记录
        //SELECT DISTINCT registration_code FROM lift GROUP BY registration_code HAVING count(*) > 1
        List<Lift> liftCodes = groupByRegistrationCode();
        Judge.notNull(liftCodes);
        List<String> codes = liftCodes.stream().map(Lift::getRegistrationCode).collect(Collectors.toList());
        //根据存在重复lift的code去查询registration_code <--> id的对应关系
        List<Lift> lifts = listByRegistrationCode(codes);
        Judge.notNull(lifts);
        Map<String, List<Long>> codeIdMap = new HashMap<>();
        //sB201707197996 SB201707197996
        codes.forEach(code -> {
            List<Long> ids = lifts.stream().filter(lift -> (StrUtil.equals(code, lift.getRegistrationCode()))).map(Lift::getId).collect(Collectors.toList());
            codeIdMap.put(code, ids);
//            log.info("{} <--> {}",code,ids);
        });
        //从相同registration_code中的lift_id的集合,取出第一个id保留，剩下的删除，并且把删除的id引用update成保留的id
        Collection<List<Long>> liftIds = codeIdMap.values();
        Judge.notNull(liftIds);
        Map<Long, List<Long>> updateIds = new HashMap<>();
        StringBuffer deleteBuffer = new StringBuffer();

        deleteBuffer.append(StrUtil.format("-- 删除lift中同一个RegistrationCode下多余id的记录 :{}", DateUtil.now())).append("\n\n");
        codeIdMap.entrySet().stream().forEach(entry -> {
            List<Long> entryValue = entry.getValue();
            int size = entryValue.size();
            //sB201707197996 SB201707197996
            if (size > 1) {
                Long first = entryValue.get(0);
                List<Long> other = entryValue.subList(1, size);
                updateIds.put(first, other);
                deleteBuffer.append(StrUtil.format("-- RegCode: {}, ids: {}, 保留id: {}", entry.getKey(), entry.getValue(), first)).append("\n");
                deleteBuffer.append(StrUtil.format("DELETE FROM lift WHERE id IN ({});", StrUtil.join(",", other))).append("\n\n");
            }
        });
        //保存脚本到文件
        String sqlname = "./sql/1.delete_lift.sql";
        FileUtil.writeString(deleteBuffer.toString(), new File(sqlname), "utf-8");
        log.info("生成DELETE操作sql文件：{}",sqlname);

        StringBuffer updateBuffer = new StringBuffer();
        //需要更新lift_id的表
        List<String> updateTables = new ArrayList<>();
        updateTables.add("annual_inspection");
        updateTables.add("emergency_repair");
        updateTables.add("lift_event");
        updateTables.add("lift_fault_report");
        updateTables.add("lift_history");
        updateTables.add("maintenance_plan");
        updateTables.add("maintenance_record");
        updateTables.add("maintenance_dispose");
        updateTables.add("project_lift_relevance");
        updateTables.add("platform_company_lift_relevance");
        updateTables.add("user_lift_record");
        updateTables.add("integral_record");
//        updateTables.add("reward_lift");
        for (int i = 0; i < updateTables.size(); i++) {
            String table = updateTables.get(i);
            int num = i + 2;
            updateBuffer.append(StrUtil.format("-- 更新{}中lift_id :{}", table, DateUtil.now())).append("\n\n");
            updateIds.entrySet().forEach(entry -> {
                updateBuffer.append(StrUtil.format("UPDATE {} SET lift_id = {} WHERE lift_id IN ({});", table, entry.getKey(), StrUtil.join(",", entry.getValue()))).append("\n");
            });
            sqlname = StrUtil.format("./sql/{}.update_{}.sql",num,table);
            FileUtil.writeString(updateBuffer.toString(), new File(sqlname), "utf-8");
            log.info("生成UPDATE操作sql文件：{}",sqlname);
            //清空buffer
            updateBuffer.setLength(0);
        }
        return true;
    }

}
