package cn.com.ty.lift.business.library.service;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.PlatformCompanyLiftRelevance;
import cn.com.ty.lift.business.library.dao.entity.model.request.PlatformCompanyBatchRequest;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectOperateServiceRequest;
import cn.com.ty.lift.business.library.dao.entity.model.request.PlatformCompanyRequest;
import cn.com.ty.lift.business.library.dao.mapper.PlatformCompanyLiftRelevanceMapper;
import cn.com.ty.lift.business.maintenance.service.MaintenancePlanService;
import cn.com.ty.lift.business.maintenance.service.MaintenanceService;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.project.service.ProjectLiftRelevanceService;
import cn.com.ty.lift.business.project.service.ProjectService;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author bieao
 * @date 2019/12/5
 * @description 平台企业电梯关联业务层
 */
@Service
public class PlatformCompanyLiftRelevanceService extends ServiceImpl<PlatformCompanyLiftRelevanceMapper, PlatformCompanyLiftRelevance> {

    @Resource
    private PlatformCompanyLiftRelevanceMapper mapper;

    @Resource
    private ProjectService projectService;

    @Resource
    private LiftService liftService;

    @Resource
    private MaintenancePlanService maintenancePlanService;

    @Resource
    private MaintenanceService maintenanceService;

    @Resource
    private ProjectLiftRelevanceService projectLiftRelevanceService;


    /**
     * @param id          电梯id
     * @param mtCompanyId 公司id
     * @return PlatformCompanyLiftRelevance
     * @description 新增平台电梯、企业电梯关联表数据
     * @date 2019/12/5 2:38 PM
     */
    public PlatformCompanyLiftRelevance save(Long id, Long mtCompanyId) {
        //建立企业电梯关联关系
        PlatformCompanyLiftRelevance companyLiftEntry = new PlatformCompanyLiftRelevance();
        companyLiftEntry.setLiftId(id);
        companyLiftEntry.setMtCompanyId(mtCompanyId);
        return save(companyLiftEntry) ? companyLiftEntry : null;
    }

    /**
     * @param paramMap 电梯id，公司id
     * @return 是否存在
     * @description 根据电梯id和公司id查询该公司是否关联过此电梯
     * @date 2019/12/20 11:39 AM
     */
    public boolean findCompanyLiftExist(Map<String, Object> paramMap) {
        QueryWrapper<PlatformCompanyLiftRelevance> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PlatformCompanyLiftRelevance> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(PlatformCompanyLiftRelevance::getMtCompanyId, paramMap.get("mtCompanyId"));
        lambdaQueryWrapper.eq(PlatformCompanyLiftRelevance::getLiftId, paramMap.get("liftId"));
        return count(lambdaQueryWrapper) > 0;
    }

    /**
     * @param mtCompanyId 公司ID
     * @return 当前电梯台量
     * @description 查询维保公司下的实际电梯台量
     * @date 2019/12/12 5:16 PM
     */
    public int countLiftNum(Long mtCompanyId) {
        QueryWrapper<PlatformCompanyLiftRelevance> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PlatformCompanyLiftRelevance> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(PlatformCompanyLiftRelevance::getMtCompanyId, mtCompanyId);
        return count(lambdaQueryWrapper);
    }

    /**
     * @param mtCompanyId 公司id
     * @param projectId 项目id
     * @return List<PlatformCompanyLiftRelevance> 公司电梯关联列表
     * @description 获取公司电梯关联列表
     * @date 2020/1/11 3:06 下午
     */
    public List<PlatformCompanyLiftRelevance> getCompanyLiftList(Long mtCompanyId, Long projectId){
        //根据项目id和公司id查询电梯列表
        List<ProjectLiftRelevance> liftList = projectLiftRelevanceService.findLiftList(mtCompanyId, projectId);
        Collection<Long> list = liftList.stream().map(ProjectLiftRelevance::getRelevanceId).collect(Collectors.toList());
        return mapper.selectBatchIds(list);
    }

    /**
     * @param request liftIdList 电梯id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 服务操作
     * @date 2019/12/10 5:42 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse serviceOperate(ProjectOperateServiceRequest request) {
        String operate = request.getOperate();
        Long projectId = request.getProjectId();
        Long mtCompanyId = request.getMtCompanyId();
        Project project = new Project();
        project.setId(projectId);
        List<PlatformCompanyLiftRelevance> resultList = getCompanyLiftList(mtCompanyId,projectId);
        switch (operate) {
            //恢复服务
            case "1":
                //判断当前公司台量是否大于企业设置的台量
                boolean judgeResult = maintenanceService.judge(mtCompanyId);
                if (!judgeResult) {
                    return RestResponse.fail(MessageUtils.get("msg.limit.num"));
                }
                project.setStartDate(request.getStartDate());
                project.setEndDate(request.getEndDate());
                project.setProjectStatus(CommonEnum.ProjectStatus.IN_SERVICE.getCode());
                projectService.modify(project);
                //电梯状态设置为正常
                resultList.forEach(entry -> entry.setLiftCompanyStatus(CommonEnum.LiftStatus.NORMAL.getCode()));
                boolean result = updateBatchById(resultList);
                if (!result) {
                    return RestResponse.fail(MessageUtils.get("msg.modify.batch.fail"));
                }
                break;
            //停止服务
            case "2":
                project.setId(projectId);
                //修改项目状态为停止
                project.setProjectStatus(CommonEnum.ProjectStatus.STOP_SERVICE.getCode());
                //设置停止时间及描述
                project.setEndRemarks(request.getEndRemarks());
                project.setStopTime(request.getStopTime());
                projectService.modify(project);

                //批量停保项目下的电梯
                resultList.forEach(entry -> entry.setLiftCompanyStatus(CommonEnum.LiftStatus.STOP_INSURANCE.getCode()));
                updateBatchById(resultList);

                //根据项目id和公司id查询电梯列表
                List<ProjectLiftRelevance> liftList = projectLiftRelevanceService.findLiftList(mtCompanyId, projectId);
                Collection<Long> list = liftList.stream().map(ProjectLiftRelevance::getLiftId).collect(Collectors.toList());
                //批量清除维保计划
                maintenancePlanService.removeMaintenancePlan(mtCompanyId, list);
                break;
            //继续服务
            case "3":
                project.setEndDate(request.getEndDate());
                project.setProjectStatus(CommonEnum.ProjectStatus.IN_SERVICE.getCode());
                boolean response = projectService.modify(project);
                if (!response) return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
                break;
            default:
                break;
        }
        return RestResponse.success(null, MessageUtils.get("msg.project.operate.success"));
    }

    /**
     * @param request relevanceId 公司电梯关联表主键id
     * @param request mtCompanyId 公司id
     * @param request liftId 电梯id
     * @return 1.成功, 0.失败, 消息描述
     * @description 更新电梯状态为停保或恢复为正常, 并清除维保计划
     * @date 2019/12/5 5:24 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse stopOrRecover(PlatformCompanyRequest request) {
        PlatformCompanyLiftRelevance entry = new PlatformCompanyLiftRelevance();
        entry.setId(request.getRelevanceId());
        //恢复电梯为正常状态
        if ("0".equals(request.getOperate())) {
            Optional<Lift> lift = liftService.getOne(null, request.getLiftId());
            String liftStatus = lift.map(Lift::getLiftStatus).orElse(null);
            // 1：弃用 2：正常
            if ("1".equals(liftStatus)) return RestResponse.fail(MessageUtils.get("msg.error.lift.status"));
            entry.setLiftCompanyStatus(CommonEnum.LiftStatus.NORMAL.getCode());
            updateById(entry);
        } else {
            entry.setLiftCompanyStatus(CommonEnum.LiftStatus.STOP_INSURANCE.getCode());
            updateById(entry);
            //清除维保计划
            maintenancePlanService.cleanPlan(request.getMtCompanyId(), request.getLiftId());
        }
        return RestResponse.success(null, MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param request companyIdList 公司电梯关联表主键id列表
     * @param request companyId 公司id
     * @param request idList 电梯id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 批量更新电梯状态为停保, 并清除维保计划
     * @date 2019/12/31 9:54 AM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse batchStopInsurance(PlatformCompanyBatchRequest request) {
        //公司电梯关联表主键id列表
        List<Long> companyIdList = request.getRelevanceIdList();
        //批量查询公司下电梯数据
        List<PlatformCompanyLiftRelevance> list = mapper.selectBatchIds(companyIdList);
        if (CollUtil.isEmpty(list)) {
            return RestResponse.success();
        }
        list.forEach(entry -> entry.setLiftCompanyStatus(CommonEnum.LiftStatus.STOP_INSURANCE.getCode()));
        //批量更新公司下电梯信息为停保
        boolean result = updateBatchById(list);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.modify.batch.fail"));
        }
        //批量清除维保计划
        maintenancePlanService.removeMaintenancePlan(request.getMtCompanyId(), request.getLiftList());
        return RestResponse.success(null, MessageUtils.get("msg.modify.batch.success"));
    }

    public List<PlatformCompanyLiftRelevance> listByCompany(Long companyId) {
        QueryWrapper<PlatformCompanyLiftRelevance> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PlatformCompanyLiftRelevance> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(PlatformCompanyLiftRelevance::getMtCompanyId, companyId);
        return list(lambdaQueryWrapper);
    }

    /**
     * 根据维保公司id和电梯id查找关联记录
     * @param mtCompanyId
     * @param liftId
     * @return
     */
    public PlatformCompanyLiftRelevance findByMtCompanyAndLift(Long mtCompanyId,Long liftId){
        QueryWrapper<PlatformCompanyLiftRelevance> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PlatformCompanyLiftRelevance> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(PlatformCompanyLiftRelevance::getMtCompanyId,mtCompanyId);
        lambdaQueryWrapper.eq(PlatformCompanyLiftRelevance::getLiftId,liftId);
        //只查询一条
        lambdaQueryWrapper.last("LIMIT 1");
        return getOne(lambdaQueryWrapper);
    }
}
