package cn.com.ty.lift.business.maintenance.controller;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenancePlan;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.*;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MaintenancePlanResponse;
import cn.com.ty.lift.business.maintenance.service.MaintenancePlanService;
import cn.com.ty.lift.common.base.ExportRequest;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.time.LocalDate;
import java.util.List;

/**
 * @author bieao
 * @date 2019/12/12
 * @description 维保管理控制层
 */
@RestController
@RequestMapping("maintenance")
public class MaintenanceController {

    @Resource
    private MaintenancePlanService maintenancePlanService;

    /**
     * @param request 维保计划列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询维保计划
     * @date 2019/12/24 10:47 AM
     */
    @PostMapping("plan/list")
    public RestResponse list(@RequestBody MaintenancePlanRequest request) {
        IPage<MaintenancePlanResponse> page = maintenancePlanService.list(request);
        if (CollUtil.isEmpty(page.getRecords())) {
            return RestResponse.success();
        }
        return RestResponse.success(page, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 公司id，项目id，电梯id
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询维保计划详情列表
     * @date 2020/1/2 9:36 上午
     */
    @PostMapping("plan/detail")
    public RestResponse detailList(@Valid @RequestBody MaintenanceDetailRequest request) {
        List<MaintenancePlan> detailList = maintenancePlanService.detailList(request);
        if (CollUtil.isEmpty(detailList)) {
            return RestResponse.success();
        }
        return RestResponse.success(detailList, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 电梯列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 制定维保计划
     * @date 2019/12/16 2:36 PM
     */
    @PostMapping("generate/plan")
    public RestResponse generatePlan(@Valid @RequestBody MaintenanceGenerateRequest request) {
        return maintenancePlanService.insertBatch(request);
    }

    /**
     * @param request 修改维保计划请求
     * @return 1.成功, 0.失败, 消息描述
     * @description 修改维保计划
     * @date 2019/12/16 2:36 PM
     */
    @PostMapping("modify/plan")
    public RestResponse modifyPlan(@Valid @RequestBody MaintenanceModifyRequest request) {
        boolean result = maintenancePlanService.modifyPlan(request);
        if (result) {
            return RestResponse.success(null, MessageUtils.get("msg.modify.success"));
        } else {
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
    }

    /**
     * @param request 公司id，项目id，电梯id
     * @return 是否成功
     * @description 清空维保计划
     * @date 2019/12/17 3:14 PM
     */
    @PostMapping("batch/clean")
    public RestResponse batchCleanPlan(@Valid @RequestBody MaintenanceCleanRequest request) {
        boolean result = maintenancePlanService.removeMaintenancePlan(request.getMtCompanyId(), request.getLiftIdList());
        if (result) {
            return RestResponse.success(null, MessageUtils.get("msg.modify.success"));
        } else {
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
    }

    /**
     * @param request 导出项目数据列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 导出维保计划
     * @date 2019/12/19 11:28 AM
     */
    @PostMapping("export/plan")
    public void export(@RequestBody ExportRequest request) {
        maintenancePlanService.export(request);
    }
}
