package cn.com.ty.lift.business.maintenance.service;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.service.LiftService;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceRecord;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceAddRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MaintenanceRecordResponse;
import cn.com.ty.lift.business.maintenance.dao.mapper.MaintenanceRecordMapper;
import cn.com.ty.lift.common.utils.MapHelper;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Optional;

/**
 * @author bieao
 * @date 2019/12/12
 * @description 保养记录业务层
 */
@Service
public class MaintenanceRecordService extends ServiceImpl<MaintenanceRecordMapper, MaintenanceRecord> {

    @Resource
    private LiftService liftService;

    /**
     * @param request 公司id和电梯id
     * @return RestResponse 保养信息
     * @description 根据公司id和电梯id查询上次保养信息
     * @date 2020/1/6 5:37 下午
     */
    public MaintenanceRecordResponse recordInfo(MaintenanceRecordRequest request) {
        return baseMapper.findRecordByCompanyId(request);
    }

    /**
     * @param id 电梯id
     * @return 电梯经纬度
     * @description 根据电梯id获取经纬度
     * @date 2020/1/13 11:37 上午
     */
    public String getCoordinate(Long id){
        Lift lift = liftService.getById(id);
        return Optional.ofNullable(lift).map(Lift::getCoordinate).orElse(null);
    }


    /**
     * @param coordinate 维保工经纬度
     * @return 是否在范围内
     * @description 计算维保工开始保养时，是否在电梯200米范围内
     * @date 2020/1/13 11:30 上午
     */
    public boolean calc(Long id, String coordinate) {
        String liftCoordinate = getCoordinate(id);
        double expected = 200;
        double result = MapHelper.GetPointDistance(liftCoordinate, coordinate);
        return (result * 1000) > expected;
    }

    /**
     * @param request 保养单信息
     * @return 是否成功
     * @description 保存保养记录
     * @date 2020/1/13 1:47 下午
     */
    public RestResponse saveRecord(MaintenanceAddRecordRequest request) {
        MaintenanceRecord record = request.getRecord();
        boolean result = calc(record.getLiftId(), request.getCoordinate());
        if (result) return RestResponse.fail(MessageUtils.get("msg.error.person.scope"));
        boolean ret = save(record);
        if (!ret) {
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.add.success"));
    }

    /**
     * @param record 保养信息
     * @return 是否成功
     * @description 编辑保养信息
     * @date 2020/1/13 2:31 下午
     */
    public boolean modify(MaintenanceRecord record){
        return updateById(record);
    }

    /**
     * @param id 保养记录id
     * @return 保养信息详情
     * @description 查看保养信息详情
     * @date 2020/1/13 2:39 下午
     */
    public MaintenanceRecord detail(Long id) {
        return getById(id);
    }
}
