package cn.com.ty.lift.business.project.controller;

import cn.com.ty.lift.business.common.Judge;
import cn.com.ty.lift.business.common.Multipart;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectImportModel;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectImportRequest;
import cn.com.ty.lift.business.project.service.ProjectService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.List;

/**
 * @author wcz
 * @date 2019/12/25
 * @description 项目导入
 */
@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("project")
public class ProjectImportController {

    private ProjectService projectService;

    /**
     * excel导入项目记录
     * @param request
     * @return
     */
    @PostMapping("import")
    public RestResponse importData(@RequestBody ProjectImportRequest request) throws Exception {
        MultipartFile file = request.getFile();
        Long mtCompanyId = request.getMtCompanyId();
        Judge.notNull(file,"没有解析到文件数据");
        Judge.id(mtCompanyId);
        // 1 检查文件有效性
        // 获取文件名，带后缀
        String originalFilename = file.getOriginalFilename();
        // 获取文件的后缀格式
        String fileSuffix = originalFilename.substring(originalFilename.lastIndexOf(".") + 1).toLowerCase();

        Judge.notTrue(!StrUtil.equalsAny(fileSuffix,"xls","xlsx"),"文件格式不支持，暂时只支持xls,xlsx");
        // 2 解析文件，转换成bean
        File excel = Multipart.multipartFileToFile(file);

        ExcelReader reader = ExcelUtil.getReader(excel);
        List<ProjectImportModel> projectImportModels = reader.read(0,2,reader.getRowCount(),ProjectImportModel.class);
        Judge.notTrue(null == projectImportModels || projectImportModels.size() < 2,"未解析到有效数据");
        // 3 循环遍历，插入数据
        return projectService.assemble(projectImportModels,mtCompanyId);
    }

    @PostMapping("parse")
    public RestResponse parse(){
        String path = "d:/tyu/项目导入模板1.xlsx";
        ExcelReader reader = ExcelUtil.getReader(path);
        Long mtCompanyId = 1l;
        // 第0行：属性名表头，第1行：汉字表头，第2行开始：记录
        List<ProjectImportModel> projectImportModels = reader.read(0,2,reader.getRowCount(),ProjectImportModel.class);
        if(ObjectUtil.isEmpty(projectImportModels)){
            return RestResponse.fail("未解析到有效数据");
        }
        //数据不超过5000条
        int size = projectImportModels.size();
        log.info("导入表格记录总条数：{}",size);
        if(size > 5000){
            return RestResponse.fail("一次性导入不能超过5000条数据");
        }
        return projectService.assemble(projectImportModels,mtCompanyId);
    }
}