package cn.com.ty.lift.business.project.controller;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.project.dao.entity.ProjectUser;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectDetailRequest;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectUserRequest;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectUserResponse;
import cn.com.ty.lift.business.project.service.ProjectUserService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.CollUtil;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * @author bieao
 * @date 2020/1/11
 * @description 项目组成员控制层
 */
@RestController
@RequestMapping("project")
public class ProjectUserController {

    @Resource
    private ProjectUserService projectUserService;

    /**
     * @param request projectId 项目ID
     * @return RestResponse userList项目组成员列表
     * @description 查询项目组成员
     * @date 2019/12/13 2:40 PM
     */
    @PostMapping("user/list")
    public RestResponse userList(@Valid @RequestBody ProjectDetailRequest request) {
        List<ProjectUserResponse> userList = projectUserService.userList(request.getId());
        if (CollUtil.isEmpty(userList)) {
            return RestResponse.success();
        }
        return RestResponse.success(userList, MessageUtils.get("msg.query.success"));
    }


    /**
     * @param request 新增项目组成员
     * @return 1.成功, 0.失败, 消息描述
     * @description 新增项目组成员
     * @date 2019/12/4 4:44 PM
     */
    @PostMapping("/user/batch/add")
    public RestResponse add(@Valid @RequestBody ProjectUserRequest request) {
        boolean result = projectUserService.add(request);
        if (result) {
            return RestResponse.success(null, MessageUtils.get("msg.add.batch.success"));
        }
        return RestResponse.fail(MessageUtils.get("msg.add.batch.fail"));
    }

    /**
     * @param projectUser 项目组成员
     * @return Integer 成功1或失败0
     * @description 修改项目组成员
     * @date 2020/1/11 10:43 上午
     */
    @PostMapping("/user/modify")
    public RestResponse modify(@RequestBody ProjectUser projectUser) {
        boolean result = projectUserService.modify(projectUser);
        if (result) {
            return RestResponse.success(null, MessageUtils.get("msg.modify.success"));
        }
        return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
    }

    /**
     * @param request 项目组成员
     * @return Integer 成功1或失败0
     * @description 删除项目组成员
     * @date 2020/1/11 10:43 上午
     */
    @PostMapping("/user/batch/remove")
    public RestResponse remove(@Valid @RequestBody ProjectUserRequest request) {
        int result = projectUserService.remove(request);
        if (result > 0) {
            return RestResponse.success(null, MessageUtils.get("msg.delete.success"));
        }
        return RestResponse.fail(MessageUtils.get("msg.delete.fail"));
    }
}
