package cn.com.ty.lift.business.project.service;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.model.request.LiftBatchMoveRequest;
import cn.com.ty.lift.business.library.dao.entity.model.request.LiftLockRequest;
import cn.com.ty.lift.business.library.dao.entity.model.request.LiftExtensionRequest;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectLiftRelevanceRequest;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectLockRequest;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectRelevanceResponse;
import cn.com.ty.lift.business.project.dao.mapper.ProjectLiftRelevanceMapper;
import cn.com.ty.lift.business.project.dao.mapper.ProjectMapper;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author bieao
 * @date 2019/12/5
 * @description 项目电梯关联业务层
 */
@Service
public class ProjectLiftRelevanceService extends ServiceImpl<ProjectLiftRelevanceMapper, ProjectLiftRelevance> {

    @Resource
    private ProjectMapper projectMapper;

    /**
     * @param entry 项目关联数据
     * @return RestResponse 状态码和返回消息
     * @description 逻辑删除电梯
     * @date 2019/12/5 2:16 PM
     */
    public RestResponse logicDelete(ProjectLiftRelevance entry) {
        entry.setDeleteFlag(1);
        boolean result = updateById(entry);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.delete.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.delete.success"));
    }

    /**
     * @param request  电梯扩展请求
     * @param id       平台企业电梯主键id
     * @param workerId 维保工id
     * @return projectLiftEntry
     * @description 新增项目电梯关联数据
     * @date 2019/12/5 2:39 PM
     */
    public boolean save(LiftExtensionRequest request, long id, long workerId) {
        Lift lift = request.getLift();
        //新增项目电梯关联表
        ProjectLiftRelevance projectLiftEntry = new ProjectLiftRelevance();
        projectLiftEntry.setProjectId(request.getProjectId());
        projectLiftEntry.setLiftId(lift.getId());
        projectLiftEntry.setRelevanceId(id);
        projectLiftEntry.setWorkerId(workerId);
        projectLiftEntry.setMtCompanyId(request.getMtCompanyId());
        return save(projectLiftEntry);
    }

    /**
     * @param list 项目电梯关联列表
     * @return 是否成功
     * @description 批量插入项目电梯关联表
     * @date 2020/1/2 2:55 下午
     */
    public boolean insertBatch(List<ProjectLiftRelevance> list) {
        return saveBatch(list, list.size());
    }

    /**
     * @param request 项目id，项目电梯关联id列表
     * @return 1.成功, 0.失败, 消息描述
     * @description 批量移动电梯
     * @date 2020/1/7 7:57 下午
     */
    public boolean move(LiftBatchMoveRequest request) {
        List<ProjectLiftRelevance> list = request.getList();
        list.forEach(entry -> entry.setProjectId(request.getProjectId()));
        return updateBatchById(list, list.size());
    }


    /**
     * @param mtCompanyId 公司id
     * @param idList      电梯id列表
     * @return 项目电梯关联列表
     * @description 根据公司id和电梯id列表查询项目电梯关联列表
     * @date 2019/12/28 1:55 PM
     */
    public List<ProjectLiftRelevance> list(Long mtCompanyId, List<Long> idList) {
        QueryWrapper<ProjectLiftRelevance> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ProjectLiftRelevance> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(ProjectLiftRelevance::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.in(ProjectLiftRelevance::getLiftId, idList);
        return list(lambdaQueryWrapper);
    }

    /**
     * @param request 电梯id列表
     * @return RestResponse 状态码和返回消息
     * @description 电梯锁定、解锁
     * @date 2019/12/5 11:00 AM
     */
    public RestResponse liftLockOrUnLock(LiftLockRequest request) {
        List<ProjectLiftRelevance> projectLiftList = list(request.getMtCompanyId(), request.getLiftList());
        if (ObjectUtil.isEmpty(projectLiftList)) {
            return RestResponse.success(null, MessageUtils.get("msg.company.lift.empty"));
        }
        projectLiftList.forEach(entry -> entry.setLiftLocked(request.getLock()));
        boolean result = updateBatchById(projectLiftList);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param mtCompanyId 公司id
     * @param projectId 项目id
     * @return List<ProjectLiftRelevance> 项目下电梯列表
     * @description 根据项目id和公司id查询项目下电梯
     * @date 2020/1/7 3:35 下午
     */
    public List<ProjectLiftRelevance> findLiftList(Long mtCompanyId, Long projectId) {
        QueryWrapper<ProjectLiftRelevance> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ProjectLiftRelevance> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(ProjectLiftRelevance::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.eq(ProjectLiftRelevance::getProjectId, projectId);
        return list(lambdaQueryWrapper);
    }

    /**
     * @param mtCompanyId 公司id
     * @param liftId      电梯id
     * @return ProjectLiftRelevance 项目电梯记录
     * @description 根据公司id和电梯id查询项目电梯记录
     * @date 2020/1/8 9:59 上午
     */
    public ProjectLiftRelevance getOne(Long mtCompanyId, Long liftId) {
        QueryWrapper<ProjectLiftRelevance> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ProjectLiftRelevance> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(ProjectLiftRelevance::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.eq(ProjectLiftRelevance::getLiftId, liftId);
        ProjectLiftRelevance entry = getOne(lambdaQueryWrapper);
        return ObjectUtil.isNotEmpty(entry) ? entry : null;
    }

    /**
     * @param relevanceId 项目电梯关联id
     * @param interval    保养间隔
     * @description 根据项目电梯关联id和保养间隔修改项目电梯关联保养间隔
     * @date 2020/1/8 9:59 上午
     */
    public void modifyPlanInterval(Long relevanceId, int interval) {
        //更新项目电梯关联表保养间隔
        ProjectLiftRelevance entry = new ProjectLiftRelevance();
        entry.setId(relevanceId);
        entry.setPlanInterval(interval);
        updateById(entry);
    }

    /**
     * @param request projectId 项目id
     * @return RestResponse 状态码和返回消息
     * @description 项目锁定、解锁,锁定该项目下的电梯
     * @date 2019/12/27 5:04 PM
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse projectLockOrUnLock(ProjectLockRequest request) {
        Project project = new Project();
        project.setId(request.getProjectId());
        project.setLocked(request.getLock());
        int ret = projectMapper.updateById(project);
        if (ret < 0) {
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        //根据项目id和公司id查询项目下电梯
        List<ProjectLiftRelevance> relevanceList = findLiftList(request.getMtCompanyId(), request.getProjectId());
        if (CollUtil.isEmpty(relevanceList)) {
            return RestResponse.success(null, MessageUtils.get("msg.project.status.update"));
        }
        relevanceList.forEach(relevance -> relevance.setLiftLocked(request.getLock()));
        boolean response = updateBatchById(relevanceList);
        if (!response) {
            return RestResponse.fail(MessageUtils.get("msg.modify.fail"));
        }
        return RestResponse.success(null, MessageUtils.get("msg.modify.success"));
    }

    /**
     * @param request 用户id
     * @return 1.成功, 0.失败, 消息描述
     * @description 查询用户下的关联项目和电梯信息
     * @date 2019/12/26 4:27 PM
     */
    public IPage<ProjectRelevanceResponse> relevanceList(ProjectLiftRelevanceRequest request) {
        IPage<ProjectRelevanceResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        return baseMapper.selectListByUserId(page, request);
    }
}
