package cn.com.ty.lift.enterprise.custom.controller;

import cn.com.ty.lift.enterprise.custom.dao.entity.PropertyCompany;
import cn.com.ty.lift.enterprise.custom.dao.entity.model.PropertyCompanyAndPropertyContactReq;
import cn.com.ty.lift.enterprise.custom.dao.entity.model.PropertyCompanyReq;
import cn.com.ty.lift.enterprise.custom.service.impl.PropertyCompanyServiceImpl;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@RestController
@RequestMapping("/property-company")
@AllArgsConstructor
/**
 * @author wang-hai-cheng
 * @date 2019/11/27
 * @description
 * 客户接口(物业公司)(甲方公司) 前端控制器
 */
public class PropertyCompanyController {
    private final PropertyCompanyServiceImpl propertyCompanyService;

    @PostMapping("/list")
    /**
     * @description 根据条件查询客户列表
     * @date 2019/12/3 8:51
     * @param [客户名称或客户地址或客户邮寄地址或客户联系方式,页码,页大小,维保公司id]
     * @return RestResponse
     */
    public RestResponse propertyCompanies(@RequestBody PropertyCompanyReq propertyCompany) {
        return propertyCompanyService.propertyCompanies(propertyCompany);
    }

    @GetMapping("/{id}")
    /**
     * @description 根据id获取客户详细信息
     * @date 2019/11/27 11:36
     * @param [客户id]
     * @return RestResponse
     */
    public RestResponse propertyCompany(@PathVariable Long id) {
        PropertyCompany byId = propertyCompanyService.getById(id);
        if (byId == null) {
            return RestResponse.success();
        }
        return RestResponse.success(byId);
    }

    @PostMapping("/add")
    /**
     * @description 新增客户
     * @date 2019/11/27 11:45
     * @param [客户]
     * @return RestResponse
     */
    public RestResponse add(@RequestBody PropertyCompanyAndPropertyContactReq req) {
        return propertyCompanyService.saveAll(req);
    }

    @PostMapping("/update")
    /**
     * @description 更新客户信息
     * @date 2019/11/27 11:45
     * @param [客户]
     * @return RestResponse
     */
    public RestResponse update(@RequestBody PropertyCompany propertyCompany) {
        if (propertyCompanyService.updateById(propertyCompany)) {
            return RestResponse.success(null, "成功");
        }
        return RestResponse.fail();
    }

    @PostMapping("/delete")
    /**
     * @description 删除客户
     * @date 2019/11/27 11:46
     * @param [客户id]
     * @return cn.com.xwy.boot.web.dto.RestResponse
     */
    public RestResponse delete(@RequestBody PropertyCompany propertyCompany) {
        return propertyCompanyService.delete(propertyCompany.getId());
    }

    @PostMapping("/project/customs")
    public RestResponse customs(@RequestBody PropertyCompany propertyCompany) {
        List<PropertyCompany> propertyCompanies = propertyCompanyService.list(new QueryWrapper<PropertyCompany>()
                .select("id", "name")
                .eq("mt_company_id", propertyCompany.getMtCompanyId()).ne("status", 0));
        if (propertyCompanies.isEmpty()) {
            return RestResponse.success();
        }
        return RestResponse.success(propertyCompanies);
    }

    /**
     * @description 项目模块，新增客户
     * @date 2019/12/16 17:08
     */
    @PostMapping("/project/custom/add")
    public RestResponse customAdd(@RequestBody PropertyCompany propertyCompany) {
        propertyCompany.setStatus(2);
        if (propertyCompanyService.save(propertyCompany)) {
            return RestResponse.success(propertyCompany.getId(), "成功");
        }
        return RestResponse.fail();
    }

    @PostMapping("/export")
    public void export(@RequestBody PropertyCompany propertyCompany, HttpServletResponse response) {
        propertyCompanyService.export(propertyCompany, response);
    }
}