package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.enterprise.common.CountPage;
import cn.com.ty.lift.enterprise.common.Judge;
import cn.com.ty.lift.enterprise.oa.dto.AnnouncementRequest;
import cn.com.ty.lift.enterprise.oa.dto.AnnouncementResponse;
import cn.com.ty.lift.enterprise.oa.entity.Announcement;
import cn.com.ty.lift.enterprise.oa.service.AnnouncementImgService;
import cn.com.ty.lift.enterprise.oa.service.AnnouncementService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;


/**
 * <p>
 * 公司公告 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */

@Slf4j
@RestController
@AllArgsConstructor
@RequestMapping("/oa/announcement")
public class AnnouncementController {

    private AnnouncementService announcementService;
    private AnnouncementImgService announcementImgService;

    /**
     * @param
     * @return
     * @description 根据id查找公告
     * @date 2019/12/6 9:24
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody AnnouncementRequest request) {
        Long id = request.getId();
        Judge.id(id);
        AnnouncementResponse entity = announcementService.findByIdWithInfo(request);
        Judge.notNull(entity);
        entity.setImgs(announcementImgService.listByAnnouncement(id));
        return Judge.result(entity);
    }

    /**
     * @param
     * @return
     * @description 根据条件查找公告
     * @date 2019/12/6 9:24
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody AnnouncementRequest request) {
        IPage<AnnouncementResponse> pages = announcementService.pageByCondition(request);
        Judge.notNull(pages);
        long count = announcementService.count();
        CountPage<AnnouncementResponse> countPage = new CountPage<>(count, pages);
        return Judge.result(countPage);
    }

    /**
     * @param
     * @return
     * @description 添加公告
     * @date 2019/12/6 9:25
     */
    @PostMapping("add")
    public RestResponse add(@Valid @RequestBody Announcement entity) {
        boolean result = announcementService.add(entity);
        return Judge.result(result);
    }

    /**
     * @param
     * @return
     * @description 根据id删除公告
     * @date 2019/12/6 9:25
     */
    @PostMapping("delete")
    public RestResponse delete(@RequestBody AnnouncementRequest request) {
        Long id = request.getId();
        Judge.id(id);
        boolean result = announcementService.removeById(id);
        return Judge.result(result);

    }
}
