package cn.com.ty.lift.enterprise.oa.controller;

import cn.com.ty.lift.enterprise.common.CountPage;
import cn.com.ty.lift.enterprise.common.Judge;
import cn.com.ty.lift.enterprise.common.Values;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceRequest;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceResponse;
import cn.com.ty.lift.enterprise.oa.entity.Attendance;
import cn.com.ty.lift.enterprise.oa.entity.MaintenanceCompany;
import cn.com.ty.lift.enterprise.oa.service.AttendanceImgService;
import cn.com.ty.lift.enterprise.oa.service.AttendanceService;
import cn.com.ty.lift.enterprise.oa.service.MaintenanceCompanyService;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.time.LocalTime;

/**
 * <p>
 * 打卡签到记录 前端控制器
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("/oa/attendance")
public class AttendanceController {

    private AttendanceService attendanceService;
    private MaintenanceCompanyService maintenanceCompanyService;
    private AttendanceImgService attendanceImgService;

    /**
     * @param
     * @return
     * @description 根据id查找
     * @date 2019/12/6 9:44
     */
    @PostMapping("findOne")
    public RestResponse findOne(@RequestBody AttendanceRequest request) {
        Long id = request.getId();
        Judge.id(id);
        AttendanceResponse entity = attendanceService.findByIdWithInfo(request);
        Judge.notNull(entity);
        entity.setImgs(attendanceImgService.listByAttendance(id));
        return Judge.result(entity);
    }

    /**
     * @param
     * @return
     * @description 根据条件查询
     * @date 2019/12/6 9:44
     */
    @PostMapping("list")
    public RestResponse list(@RequestBody AttendanceRequest request) {
        IPage<AttendanceResponse> pages = attendanceService.pageByCondition(request);
        Judge.notNull(pages);
        long count = attendanceService.count();
        CountPage<AttendanceResponse> countPage = new CountPage<>(count, pages);
        return Judge.result(countPage);
    }

    /**
     * @param
     * @return
     * @description 设置打卡时间
     * @date 2019/12/20 16:30
     */
    @PostMapping("setupTime")
    public RestResponse setupTime(@RequestBody AttendanceRequest request) {
        LocalTime topTime = request.getTopTime();
        LocalTime downTime = request.getDownTime();
        Judge.notTrue(ObjectUtil.hasEmpty(topTime, downTime), Values.At.SetAmTime);
        Judge.notTrue(topTime.isAfter(downTime), Values.At.InvalidAmTime);

        LocalTime topTimePm = request.getTopTimePm();
        LocalTime downTimePm = request.getDownTimePm();
        Judge.notTrue(ObjectUtil.hasEmpty(topTimePm, downTimePm), Values.At.SetPmTime);
        Judge.notTrue(topTimePm.isAfter(downTimePm), Values.At.InvalidPmTime);

        LocalTime topTimeOt = request.getTopTimeOt();
        LocalTime downTimeOt = request.getDownTimeOt();
        Judge.notTrue(ObjectUtil.hasEmpty(topTimeOt, downTimeOt), Values.At.SetOtTime);
        Judge.notTrue(topTimeOt.isAfter(downTimeOt), Values.At.InvalidOtTime);

        Long mtCompanyId = request.getMtCompanyId();
        Judge.id(mtCompanyId);
        MaintenanceCompany entity = maintenanceCompanyService.getById(mtCompanyId);
        Judge.notNull(entity);

        entity.setTopTime(topTime);
        entity.setDownTime(downTime);
        entity.setTopTimePm(topTimePm);
        entity.setDownTimePm(downTimePm);
        entity.setTopTimeOt(topTimeOt);
        entity.setDownTimeOt(downTimeOt);

        boolean result = maintenanceCompanyService.saveOrUpdate(entity);
        return Judge.result(result);
    }

    /**
     * 打卡
     * 11 上午上班 12 上午下班
     * 21 下午上班 22 下午下班
     * 31 加班上班 32 加班下班
     *
     * @param entity
     * @return
     */
    @PostMapping("clock")
    public RestResponse clock(@Valid @RequestBody Attendance entity) {
        Long mtCompanyId = entity.getMtCompanyId();

        MaintenanceCompany maintenanceCompany = maintenanceCompanyService.getById(mtCompanyId);
        Judge.notNull(maintenanceCompany);

        LocalTime now = LocalTime.now();
        int type = entity.getType();
        //状态（是否迟到早退，0：否，1：是）
        switch (type) {
            case 11:
                LocalTime topTime = maintenanceCompany.getTopTime();
                Judge.notNull(topTime, Values.At.SetAmTime);
                //打卡时间比上班时间早-正常，否则迟到
                entity.setStatus(now.isBefore(topTime) ? 0 : 1);
                entity.setRemarks(Values.At.AmTopClock);
                break;
            case 12:
                LocalTime downTime = maintenanceCompany.getDownTime();
                Judge.notNull(downTime, Values.At.SetAmTime);
                //打卡时间比下班时间晚-正常，否则早退
                entity.setStatus(now.isAfter(downTime) ? 0 : 1);
                entity.setRemarks(Values.At.AmDownClock);
                break;
            case 21:
                LocalTime topTimePm = maintenanceCompany.getTopTimePm();
                Judge.notNull(topTimePm, Values.At.SetPmTime);
                entity.setStatus(now.isBefore(topTimePm) ? 0 : 1);
                entity.setRemarks(Values.At.PmTopClock);
                break;
            case 22:
                LocalTime downTimePm = maintenanceCompany.getDownTimePm();
                Judge.notNull(downTimePm, Values.At.SetPmTime);
                entity.setStatus(now.isAfter(downTimePm) ? 0 : 1);
                entity.setRemarks(Values.At.PmDownClock);
                break;
            case 31:
                LocalTime topTimeOt = maintenanceCompany.getTopTimeOt();
                Judge.notNull(topTimeOt, Values.At.SetOtTime);
                entity.setStatus(now.isBefore(topTimeOt) ? 0 : 1);
                entity.setRemarks(Values.At.OtTopClock);
                break;
            case 32:
                LocalTime downTimeOt = maintenanceCompany.getDownTimeOt();
                Judge.notNull(downTimeOt, Values.At.SetOtTime);
                entity.setStatus(now.isAfter(downTimeOt) ? 0 : 1);
                entity.setRemarks(Values.At.OtDownClock);
                break;
            default:
                return RestResponse.failParam();
        }
        boolean result = attendanceService.save(entity);
        return Judge.result(result);
    }

    /**
     * 返回维保公司打卡时间信息
     *
     * @param request
     * @return
     */
    @PostMapping("time")
    public RestResponse time(@RequestBody AttendanceRequest request) {
        Long mtCompanyId = request.getMtCompanyId();
        Judge.id(mtCompanyId);
        MaintenanceCompany entity = maintenanceCompanyService.getById(mtCompanyId);
        return Judge.result(entity);
    }

    /**
     * 根据用户查询打卡记录
     *
     * @param request
     * @return
     */
    @PostMapping("pageByUser")
    public RestResponse pageByUser(@RequestBody AttendanceRequest request) {
        Long userId = request.getUserId();
        Judge.id(userId);
        IPage<AttendanceResponse> pages = attendanceService.pageByUser(request);
        return Judge.result(pages);
    }
}
