package cn.com.ty.lift.enterprise.oa.service;

import cn.com.ty.lift.enterprise.oa.dto.AttendanceRequest;
import cn.com.ty.lift.enterprise.oa.dto.AttendanceResponse;
import cn.com.ty.lift.enterprise.oa.entity.Attendance;
import cn.com.ty.lift.enterprise.oa.mapper.AttendanceMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 打卡签到记录 服务类
 * </p>
 *
 * @author wcz
 * @since 2019-12-12
 */
@Service
public class AttendanceService extends ServiceImpl<AttendanceMapper,Attendance>{

    /**
     * 根据条件分页查找签到记录
     * @param request
     * @return
     */
    public IPage<AttendanceResponse> pageByCondition(AttendanceRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<AttendanceResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByCondition(page, request);
    }

    /**
     * 根据用户分页查找记录
     * @param request
     * @return
     */
    public IPage<AttendanceResponse> pageByUser(AttendanceRequest request) {
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
        IPage<AttendanceResponse> page = new Page<>(pageNum, pageSize);
        return baseMapper.pageByUser(page, request);
    }

    /**
     * 根据用户和打卡类型统计打卡记录条数
     * @param mtCompanyId
     * @param userId
     * @param type
     * @return
     */
    public int countByUserAndType(Long mtCompanyId,Long userId,int type){
        QueryWrapper<Attendance> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<Attendance> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(Attendance::getMtCompanyId,mtCompanyId);
        lambdaQueryWrapper.eq(Attendance::getUserId,userId);
        lambdaQueryWrapper.eq(Attendance::getType,type);
        return count(lambdaQueryWrapper);
    }

    /**
     * 根据id查询，带关联信息
     * @param request
     * @return
     */
    public AttendanceResponse findByIdWithInfo(AttendanceRequest request){
        return baseMapper.findByIdWithInfo(request);
    }
}