package cn.com.ty.lift.enterprise.oa.service;

import cn.com.ty.lift.enterprise.oa.dto.LiftCertificateRequest;
import cn.com.ty.lift.enterprise.oa.dto.LiftCertificateResponse;
import cn.com.ty.lift.enterprise.oa.entity.LiftCertificate;
import cn.com.ty.lift.enterprise.oa.mapper.LiftCertificateMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
/**
 * <p>
 * 操作证 服务类
 * </p>
 *
 * @author wcz
 * @since 2019-12-04
 */
@Service
public class LiftCertificateService extends ServiceImpl<LiftCertificateMapper,LiftCertificate>{

    /**
     * 根据条件分页查询操作证记录
     * @param request
     * @return
     */
    public IPage<LiftCertificateResponse> pageByCondition(LiftCertificateRequest request){
        long pageNum = request.getPageNum();
        long pageSize = request.getPageSize();
		IPage<LiftCertificateResponse> page = new Page<>(pageNum,pageSize);
        return baseMapper.pageByCondition(page,request);
    }

    /**
     * 根据条件总计操作证记录条数
     * @param request
     * @return
     */
    public long countByCondition(LiftCertificateRequest request){
        return baseMapper.countByCondition(request);
    }

    /**
     * 根据用户id和维保公司统计操作证条数
     * @param userId
     * @param mtCompanyId
     * @return
     */
    public int countByUserAndMtCompany(Long userId,Long mtCompanyId){
        QueryWrapper<LiftCertificate> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<LiftCertificate> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(LiftCertificate::getOwnerId,userId);
        lambdaQueryWrapper.eq(LiftCertificate::getMtCompanyId, mtCompanyId);
        return count(lambdaQueryWrapper);
    }

    /**
     * 根据id和用户，维保公司查询
     * @param id
     * @param userId
     * @param mtCompanyId
     * @return
     */
    public LiftCertificate findByUserAndMtCompany(Long id,Long userId,Long mtCompanyId){
        QueryWrapper<LiftCertificate> queryWrapper = new QueryWrapper<>();
        LambdaQueryWrapper<LiftCertificate> lambdaQueryWrapper = queryWrapper.lambda();
        lambdaQueryWrapper.eq(LiftCertificate::getId,id);
        lambdaQueryWrapper.eq(LiftCertificate::getOwnerId,userId);
        lambdaQueryWrapper.eq(LiftCertificate::getMtCompanyId, mtCompanyId);
        lambdaQueryWrapper.last("LIMIT 1");
        return getOne(lambdaQueryWrapper);
    }

    /**
     * 根据id查询操作证，带关联信息
     * @param request
     * @return
     */
    public LiftCertificateResponse findByIdWithInfo(LiftCertificateRequest request){
        return baseMapper.findByIdWithInfo(request);
    }
}