package cn.com.ty.lift.enterprise.option.controller;


import cn.com.ty.lift.enterprise.option.dao.entity.MaintenanceOption;
import cn.com.ty.lift.enterprise.option.dao.entity.model.OptionReq;
import cn.com.ty.lift.enterprise.option.service.impl.MaintenanceOptionServiceImpl;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 * 保养项目 前端控制器
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-12-18
 */
@RestController
@AllArgsConstructor
@RequestMapping("/maintenance-option")
public class MaintenanceOptionController {
    private final MaintenanceOptionServiceImpl optionService;

    /**
     * @param req 页码 页大小
     * @return cn.com.xwy.boot.web.dto.RestResponse
     * @description 分页获取维保项列表
     * @date 2019/12/19 9:21
     */
    @PostMapping("/list")
    public RestResponse list(@RequestBody OptionReq req) {
        IPage<MaintenanceOption> page = optionService.page(
                new Page<>(req.getPageNum(), req.getPageSize()), new QueryWrapper<MaintenanceOption>()
                        .in("mt_company_id", 0L, req.getMtCompanyId())
                        .eq(req.getType() != null, "type", req.getType())
                        .eq(req.getLiftType() != null, "lift_type", req.getLiftType())
                        .orderByDesc("mt_company_id")
        );
        if (page.getRecords().isEmpty()) {
            return RestResponse.success();
        }
        return RestResponse.success(page);
    }

    @PostMapping("/options")
    public RestResponse options(@RequestBody MaintenanceOption req) {
        List<MaintenanceOption> options = optionService.list(
                new QueryWrapper<MaintenanceOption>()
                        .in("mt_company_id", 0L, req.getMtCompanyId())
                        .eq(req.getType() != null, "type", req.getType())
                        .eq(req.getLiftType() != null, "lift_type", req.getLiftType())
                        .eq(req.getSort() != null, "sort", req.getSort()));
        if (options.isEmpty()) {
            return RestResponse.success();
        }
        return RestResponse.success(options);
    }

    /**
     * @param option 维保项信息
     * @return RestResponse
     * @description 新增维保项
     * @date 2019/12/19 9:20
     */
    @PostMapping("/add")
    public RestResponse add(@RequestBody MaintenanceOption option) {
        if (option.getMtCompanyId().equals(0L)) {
            option.setEdit(0);
        }
        if (optionService.save(option)) {
            return RestResponse.success(option.getId(), "成功");
        }
        return RestResponse.fail();
    }

    /**
     * @param option 维保项信息
     * @return RestResponse
     * @description 判断是否默认维保项，不是默认维保项才可以更新
     * @date 2019/12/19 10:03
     */
    @PostMapping("/modify")
    public RestResponse modify(@RequestBody MaintenanceOption option) {
        MaintenanceOption byId = optionService.getById(option.getId());
        if (byId.getEdit() == 0) {
            return RestResponse.fail("默认维保项无法编辑");
        }
        if (optionService.updateById(option)) {
            return RestResponse.success(null, "成功");
        }
        return RestResponse.failParam();
    }

    /**
     * @param option 维保项id
     * @return RestResponse
     * @description 判断是否是默认维保项，不是默认的维保项才可以删除
     * @date 2019/12/19 9:20
     */
    @PostMapping("/delete")
    public RestResponse delete(@RequestBody MaintenanceOption option) {
        MaintenanceOption byId = optionService.getById(option.getId());
        if (byId.getEdit() == 0) {
            return RestResponse.fail("默认维保项无法删除");
        }
        if (optionService.removeById(option.getId())) {
            return RestResponse.success(null, "成功");
        }
        return RestResponse.fail();
    }
}