package cn.com.ty.lift.push.app;

import cn.hutool.core.date.DateUtil;
import com.tencent.xinge.XingeApp;
import com.tencent.xinge.bean.AudienceType;
import com.tencent.xinge.bean.Message;
import com.tencent.xinge.bean.MessageType;
import com.tencent.xinge.bean.Platform;
import com.tencent.xinge.push.app.PushAppRequest;
import lombok.extern.slf4j.Slf4j;
import org.json.JSONObject;

import java.util.ArrayList;

/**
 * @author wcz
 * @date 2019/12/17
 * @description 腾讯信鸽推动 https://xg.qq.com/docs/server_api/v3/push_api_v3.html
 */
@Slf4j
public class XingePush {

    private static final String appId = "40d4e2e854433";
    private static final String secretKey = "fa99dc19157b00fc71cf8d7e840273a3";

    private static final String access = "18571500256";

    public static void main(String[] args) {
        ArrayList<String> account_list = new ArrayList<>();
        account_list.add(access);
        MsgRequest request = MsgRequest.android("天域电梯测试","测试信鸽消息推送，当前时间: " + DateUtil.now(),account_list);

        pushMessage(request);
    }
    /**
     * 消息推送(单推)
     */
    public static Boolean pushMessage(MsgRequest msgRequest) {
        //消息推送（>>>）
        XingeApp xingeApp = new XingeApp(appId, secretKey);
        /**
         * https://xg.qq.com/docs/server_api/v3/push_api_v3.html#message%EF%BC%9A%E6%B6%88%E6%81%AF%E4%BD%93
         */
        Message message = new Message();
        message.setTitle(msgRequest.getTitle());
        message.setContent(msgRequest.getContent());
        //推送请求信息
        PushAppRequest pushAppRequest = new PushAppRequest();
        //消息体
        pushAppRequest.setMessage(message);
        /**
         * 消息类型
         1）notify：通知
         2）message：透传消息/静默消息
         */
        pushAppRequest.setMessage_type(MessageType.notify);
        /**
         * 客户端平台类型
         1）android：安卓
         2）ios：苹果
         */
        if(msgRequest.getPlatform() == 1){
            pushAppRequest.setPlatform(Platform.android);
        }else if(msgRequest.getPlatform() == 2){
            pushAppRequest.setPlatform(Platform.ios);
        }else{
            return false;
        }
        /**
         * 推送目标
         1）all：全量推送
         2）tag：标签推送
         3）token：单设备推送
         4）token_list：设备列表推送
         5）account：单账号推送
         6）account_list：账号列表推送
         */

        if(msgRequest.getAllPlatform()){
            pushAppRequest.setAudience_type(AudienceType.all);
        }else {
            ArrayList<String> accountList = msgRequest.getAccountList();
            if(null == accountList || accountList.size() == 0){
                return false;
            }
            if(accountList.size() == 1){
                pushAppRequest.setAudience_type(AudienceType.account);
            }else{
                pushAppRequest.setAudience_type(AudienceType.account_list);
            }
            //推送账号
            pushAppRequest.setAccount_list(msgRequest.getAccountList());
        }

        //打印下推送数据
        /**
         * {
         * 	"audience_type": "account",
         * 	"platform": "android",
         * 	"message_type": "notify",
         * 	"message": {
         * 		"title": "天域电梯测试",
         * 		"content": "测试信鸽消息推送，当前时间: 2019-12-24 17:03:15",
         * 		"accept_time": null,
         * 		"android": null,
         * 		"ios": null
         *        },
         * 	"expire_time": 259200,
         * 	"send_time": null,
         * 	"multi_pkg": false,
         * 	"loop_times": 0,
         * 	"loop_interval": 0,
         * 	"environment": "product",
         * 	"stat_tag": null,
         * 	"seq": 0,
         * 	"tag_list": null,
         * 	"account_list": ["18571500256"],
         * 	"account_type": 0,
         * 	"token_list": [],
         * 	"push_id": null
         * }
         */
        //推送结果 0是推送成功
        /**
         * {
         * 	"result": "[0]",
         * 	"environment": "product",
         * 	"push_id": "1079290952",
         * 	"err_msg": "",
         * 	"ret_code": 0,
         * 	"seq": 0
         * }
         */
        String request = pushAppRequest.toString();
        log.info("信鸽推送消息体: {}",request);
        JSONObject ret = xingeApp.pushApp(request);
        log.info("信鸽推送消息返回: {}",ret);
        return true;
    }
}
