package cn.com.ty.lift.quan.config;

import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.json.JSONUtil;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * @author bieao
 * @date 2019/12/3 10:30 AM
 * @description 日志切面
 */
@Slf4j
@Aspect
@Component
public class ControllerAspect {

    private static final String line = "============================================================";
    private static final String head = "#####| ";
    private static final String url = head + "URL          : ";
    private static final String httpMethod = head + "HTTP_METHOD  : ";
    private static final String ip = head + "IP           : ";
    private static final String classMethod = head + "CLASS_METHOD : ";
    private static final String args = head + "ARGS         : ";
    private static final String resp = head + "RESPONSE     : ";

    @Pointcut("execution(* cn.com.ty.lift.quan.*.controller..*(..))")
    public void controllerPointCut() {
    }

    @Before("controllerPointCut()")
    public void doBefore(JoinPoint joinPoint) throws Throwable {
        // Receives the request and get request content
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        log.info(url + request.getRequestURL().toString());
        log.info(httpMethod + request.getMethod());
        log.info(ip + request.getRemoteAddr());
        log.info(classMethod + joinPoint.getSignature().getDeclaringTypeName() + "." + joinPoint.getSignature().getName());
        log.info(args + Arrays.toString(joinPoint.getArgs()));

    }

    @AfterReturning(returning = "response", pointcut = "controllerPointCut()")
    public void doAfterReturning(RestResponse response) throws Throwable {
        // Processes the request and returns the content
        log.info(resp + JSONUtil.parse(response));
        log.info(line);
    }

    @AfterThrowing(throwing = "ex", pointcut = "controllerPointCut()")
    public void doAfterThrowing(QuanBasicException ex) throws Throwable {
        final String msg = ex.getMessage();
        // Processes the request and returns the content
        log.info(resp +  ex.addSuffix(msg).replace("()", ""));
        log.info(line);
    }


    @Around("controllerPointCut()")
    public Object interceptor(ProceedingJoinPoint proceedingJoinPoint) throws Throwable {
        log.info(line);
        return proceedingJoinPoint.proceed();
    }

}
