package cn.com.ty.lift.quan.pay.controller;


import cn.com.ty.lift.quan.pay.dao.entity.UserInfo;
import cn.com.ty.lift.quan.pay.dao.entity.model.PayReq;
import cn.com.ty.lift.quan.pay.service.impl.UserInfoServiceImpl;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.alipay.api.AlipayApiException;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * <p>
 * 用户资金表 前端控制器
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-12-26
 */
@RestController
@RequestMapping("/pay")
@AllArgsConstructor
public class PayController {
    private final UserInfoServiceImpl payService;

    /**
     * @param userInfo 用户id，支付宝账号
     * @return 操作结果
     * @description 绑定/换绑支付宝账号
     * @date 2019/12/26 14:28
     */
    @PostMapping("/bind")
    public RestResponse bind(@RequestBody UserInfo userInfo) {
        return payService.bind(userInfo);
    }

    /**
     * @param userInfo 用户id
     * @return 操作结果
     * @description 用户信息
     * @date 2020/01/06 09:41
     */
    @PostMapping("/info")
    public RestResponse balance(@RequestBody UserInfo userInfo) {
        UserInfo byId = payService.getById(userInfo.getUserId());
        if (byId == null) {
            return RestResponse.success("无此用户");
        }
        return RestResponse.success(byId);
    }

    /**
     * @param req task 1创建充值订单 2提现审核订单,userId,
     * @return 操作结果
     * @description 创建订单
     * @date 2019/12/26 16:05
     */
    @PostMapping("/create")
    public RestResponse pay(@RequestBody PayReq req) {
        return payService.pay(req);
    }

    /**
     * @return 操作结果
     * @description 审核订单
     * @date 2019/12/30 14:03
     * // @param 商家自定义订单号
     */
    @PostMapping("/order")
    public RestResponse order(@RequestBody PayReq req) throws AlipayApiException {
        return payService.order(req);
    }

    /**
     * @return 操作结果
     * @description 取消订单
     * @date 2019/12/30 14:05
     * //@param 订单号
     */
    @PostMapping("/cancel")
    public RestResponse cancel(@RequestBody PayReq req) throws AlipayApiException {
        return payService.cancel(req);
    }

    @PostMapping("/order/list")
    public RestResponse orders(@RequestBody PayReq req) {
        return payService.orders(req);
    }
}