package cn.com.ty.lift.system.faq.controller;


import cn.com.ty.lift.system.faq.dao.entity.Faq;
import cn.com.ty.lift.system.faq.service.impl.FaqServiceImpl;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author wang-hai-cheng
 * @date 2019/12/4
 * @description 常见问题级常见问题分类表
 */
@RestController
@RequestMapping("/faq")
@AllArgsConstructor
public class FaqController {
    private final FaqServiceImpl faqService;

    /**
     * @return RestResponse
     * @description 获取分类及问题列表
     * @date 2019/12/13 10:11
     */
    @PostMapping("/child")
    public RestResponse child() {
        List<Faq> faqs = faqService.list(new QueryWrapper<Faq>()
                .in("type", 1, 2)
                .orderByAsc("parent_id", "type", "serial"));
        if (faqs.isEmpty()) {
            return RestResponse.success();
        }
        return RestResponse.success(faqs, "成功");
    }

    @PostMapping("/list")
    /**
     * @description 查询分类或问题或答案
     * @date 2019/11/27 10:19
     * @param faq type和parentId
     * @return RestResponse
     */
    public RestResponse one(@RequestBody Faq faq) {
        return faqService.getFaqs(faq);
    }

    @PostMapping("/add")
    /**
     * @description 新增分类或问题或答案
     * @date 2019/11/27 10:47
     * @param [faq]
     * @return RestResponse
     */
    public RestResponse add(@RequestBody Faq faq) {
        return faqService.add(faq);
    }


    /**
     * @param faq 需要更新的数据
     * @return RestResponse
     * @description 更新分类名或问题标题或答案内容, 都是title参数,
     * description是对此id信息的描述
     * @date 2019/11/27 10:51
     */
    @PostMapping("/update")
    public RestResponse updateOne(@RequestBody Faq faq) {
        if (faqService.updateById(faq)) {
            return RestResponse.success(null, "成功");
        }
        return RestResponse.fail();
    }

    @PostMapping("/delete")
    /**
     * @description 根据id删除此id下的分类和问答
     * @date 2019/11/27 10:51
     * @param [id] 要删除的id
     * @return RestResponse
     */
    public RestResponse delete(@RequestBody Faq faq) {
        return faqService.delete(faq.getId());
    }

    @PostMapping("/up")
    /**
     * @description 排序上移一位
     * @date 2019/11/27 10:52
     * @param [id]
     * @return cn.com.xwy.boot.web.dto.RestResponse
     */
    public RestResponse up(@RequestBody Faq faq) {
        return faqService.serial(faq.getId(), 1);
    }

    @PostMapping("/down")
    /**
     * @description 排序下移一位
     * @date 2019/11/27 10:52
     * @param [id]
     * @return cn.com.xwy.boot.web.dto.RestResponse
     */
    public RestResponse down(@RequestBody Faq faq) {
        return faqService.serial(faq.getId(), 0);
    }
}