package cn.com.ty.lift.system.faq.service.impl;

import cn.com.ty.lift.system.faq.dao.entity.Faq;
import cn.com.ty.lift.system.faq.dao.mapper.FaqMapper;
import cn.com.ty.lift.system.faq.service.FaqService;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

/**
 * <p>
 * 常见问题级常见问题分类表 服务实现类
 * </p>
 *
 * @author wang-hai-cheng
 * @since 2019-12-05
 */
@Service
public class FaqServiceImpl extends ServiceImpl<FaqMapper, Faq> implements FaqService {

    public RestResponse getFaqs(Faq faq) {
        List<Faq> faqs = this.list(new QueryWrapper<Faq>()
                .eq(faq.getType() != null, "type", faq.getType())
                .eq(faq.getParentId() != null, "parent_id", faq.getParentId())
                .orderByAsc("serial"));
        if (faqs.isEmpty()) {
            return RestResponse.success();
        }
        return RestResponse.success(faqs, "成功");
    }

    /**
     * @description 新增分类或问题或答案
     * @date 2019/12/5 11:01
     */
    public RestResponse add(Faq faq) {
        List<Faq> faqs = this.list(new QueryWrapper<Faq>()
                .select("id", "serial")
                .eq("type", faq.getType())
                .eq("parent_id", faq.getParentId())
                .orderByDesc("serial"));
        faq.setSerial(faqs.size() + 1);
        if (this.save(faq)) {
            return RestResponse.success(null, "成功");
        }
        return RestResponse.fail();
    }

    public RestResponse delete(Long id) {
        List<Long> ids = new ArrayList<>();
        List<Faq> faqs = this.list(new QueryWrapper<Faq>().select("id", "parent_id"));
        this.ids(id, faqs, ids);
        ids.add(id);
        if (this.removeByIds(ids)) {
            return RestResponse.success(null, "成功");
        }
        return RestResponse.fail();
    }

    private void ids(Long id, List<Faq> faqs, List<Long> ids) {
        List<Long> childIds = new ArrayList<>();
        //遍历常见问题列表
        for (Faq faq : faqs) {
            //如果是要删除id下的子id
            if (id.equals(faq.getParentId())) {
                //就把id标记删除
                childIds.add(faq.getId());
                //然后再遍历这个子id，看有没有符合的id要标记删除，并将要删除的id列表传进去
                this.ids(faq.getId(), faqs, ids);
            }
        }
        //把要删除的子id记录到ids列表里
        ids.addAll(childIds);
    }

    //type 1上移 0下移
    public RestResponse serial(Long id, Integer type) {
        Faq byId = this.getById(id);
        if (null == byId) {
            return RestResponse.success();
        }
        List<Faq> faqs = this.list(new QueryWrapper<Faq>()
                .eq("parent_id", byId.getParentId())
                .eq("type", byId.getType())
                .orderByDesc("serial"));
        if (faqs.size() == 1) {
            return RestResponse.success(null, "成功");
        }
        //type 0下移
        //type 1上移
        if (type == 1) {
            faqs.sort(Comparator.comparing(Faq::getSerial));
        }
        if (faqs.get(0).getSerial().equals(byId.getSerial())) {
            return RestResponse.success(null, "成功");
        }
        if (upOrDown(byId, faqs)) {
            return RestResponse.success(null, "成功");
        }
        return RestResponse.fail();
    }

    private boolean upOrDown(Faq byId, List<Faq> faqs) {
        System.out.println(faqs);
        Faq f = new Faq();
        for (Faq faq : faqs) {
            if (faq.getSerial().equals(byId.getSerial())) {
                Integer serial = f.getSerial();
                f.setSerial(byId.getSerial());
                byId.setSerial(serial);
                if (this.updateById(byId) && this.updateById(f)) {
                    return true;
                }
            }
            f = faq;
        }
        return false;
    }
}